#include "stdafx.h"
#include "EventQueue.h"
#include "MsSong.h"
#include "MsObject.h"
#include "WindowMessages.h"
#include "MsBar.h"
#include "MsEnd.h"
#include "MsKeySignature.h"
#include "MsLoudness.h"
#include "MsNote.h"
#include "MsObject.h"
#include "MsRepeatEnd.h"
#include "MsRepeatStart.h"
#include "Mstempo.h"
#include "MsTimeSig.h"
#include "MusicStudioPlayer.h"

CEventQueue::CEventQueue()
{
    ///---------------------------
    /// Initialize data members
    ///---------------------------
    m_EventHead = 0;
    m_EventTail = 0;
	m_TotalEvents = 0;
	m_Tempo = 30;
	m_Loudness = 127;
	m_pRepeatStack = 0;
	int i;
	for(i=0;i< 16;++i)
		m_Patches[i] = (char)0xff;
}

CEventQueue::~CEventQueue()
{
    //dtor
	if(m_EventHead)
	{
		CEventMidi *pEM,*pEMdel;

		pEM = m_EventHead;
		while(pEM)
		{
			pEMdel = pEM;
			pEM = pEM->GetNext();
			delete pEMdel;
		}
	}
	if(m_pRepeatStack)
	{
		delete m_pRepeatStack;
	}
}

void CEventQueue::AddEvent(CEventMidi *pEM)
{
    ///---------------------------------------
    /// AddEvent
    ///     this method adds a new event to
    /// the queue to be processed.
    /// parameters:
    ///     pEM.....pointer to event to add
    ///
    ///----------------------------------------

	++m_TotalEvents;
    if(m_EventHead == 0)
    {
        m_EventHead = pEM;
        m_EventTail = pEM;
    }
    else
    {
        /// add new event to the tail
        pEM->SetPrev(m_EventTail);
        m_EventTail->SetNext(pEM);
        m_EventTail = pEM;
    }
}

void CEventQueue::DeleteEvent(CEventMidi *pEM)
{
	///---------------------------------
	/// De;eteEvent
	///		This function deletesan event
	/// from the queue.
	///
	/// parameters:
	///		pEM.....Event to delete
	///---------------------------------
	--m_TotalEvents;	///decement event count
    if(pEM->GetPrev() == 0)
    {
		if(pEM->GetNext())
		{
			m_EventHead = pEM->GetNext();
			m_EventHead->SetPrev(0);
		}
		else
		{
			//-----------------------
			// All Gone
			//-----------------------
			m_EventHead = 0;
			m_EventTail = 0;
		}
    }
    else if(pEM->GetNext() == 0)
    {
        m_EventTail = pEM->GetPrev();
        m_EventTail->SetNext(0);
    }
    else
    {
        pEM->GetPrev()->SetNext(pEM->GetNext());
        pEM->GetNext()->SetPrev(pEM->GetPrev());
    }
    delete pEM;
}

extern char *NoteLUT[12];


void CEventQueue::Start(void)
{
	m_MidiClockFlag = 1;
	m_StopFlag = 0;
	m_TempoCount = 0;
	m_SongDone = 0;
	MidiStart();
}

int CEventQueue::Stop()
{
	m_StopFlag = 1;
	return 0;
}

int CEventQueue::PendingEvents()
{
	//------------------------------------
	// PendingEvents
	//	Returns 0 if there are no
	// pending events
	//-------------------------------------
	int rV = 0;
	FILE *m_pLog;
	m_pLog = ((CMSPlayerApp *)AfxGetApp())->GetLogFile();
	CEventMidi *pCEM;
	pCEM = this->m_EventHead;
	while (pCEM)
	{
		++rV;
		if (m_pLog)
		{
			int rest = pCEM->GetNote()->IsRest();
			fprintf(m_pLog, "%s  %2x  Ticks=%d  Beg:%d End:%d\n", rest ? "Rest" : "Note",
				pCEM->GetNote()->GetPitch(),
				pCEM->GetTick(),
				pCEM->GetNote()->IsTieBeg(),
				pCEM->GetNote()->IsTieend());
		}
		
		pCEM = pCEM->GetNext();
	}
//	if (!pCEM)
//		rV = 0;
	if (m_pLog)
		fprintf(m_pLog, "Pending Events=%d\n", rV);
	return rV;
}

int CEventQueue::Ticker(void)
{
	///-----------------------------
	/// Ticker
	///		This is the function that
	///	is calleed by the multimedia
	/// time.  This is wherre the
	/// we figure out when to play
	/// the midi event
	//
	/// Also, we need to output a 
	/// Midi clock 24 time per
	/// quarter note.  A quarter
	/// not is 48 ticks, so every
	/// other time this function
	/// is called, we need to output
	/// a midi clock.
	///
	/// RETURNS:0 when done,
	///         1 while in progress
	///-----------------------------
	int rV = 1;
    int t;
    int DeleteEventFlag = 0;
	CMsNote *pN;
	CEventMidi *pP;
    CEventMidi *pEM = m_EventHead;
	if(m_MidiClockFlag)
	{
		MidiClock();
		m_MidiClockFlag = 0;
	}
	else m_MidiClockFlag = 1;

    while(pEM)
    {
	 	pN = pEM->GetNote();
		if((t  = pEM->DecTicks()))
        {
            if(t == 2)	// indicates count is 2
            {
				if(!pN->IsRest() && !pN->IsTieBeg())
					pN->NoteOff(pEM->GetVelocity(),m_pMsKeySig);
				this->m_pWnd->SendMessage(WM_PLAYER,PLAYER_SONG_TICKDEBUG,(long)pEM);
            }
            else if (t == 1)	///indicates count is 0
            {
                rV = 0;
                DeleteEventFlag = 1;
            }
			else if (t < 0)
			{
				FILE *m_pLog;
				m_pLog = ((CMSPlayerApp *)AfxGetApp())->GetLogFile();
				if (m_pLog)
				{
					int rest = pEM->GetNote()->IsRest();
					fprintf(m_pLog, "%s  %2x  Ticks=%d  Beg:%d End:%d\n", rest ? "Rest" : "Note",
						pEM->GetNote()->GetPitch(),
						pEM->GetTick(),
						pEM->GetNote()->IsTieBeg(),
						pEM->GetNote()->IsTieend());

				}
			}
        }///end of if((t  = pEM->DecTicks()))
		pP = pEM;
        pEM = pEM->GetNext();
        if(DeleteEventFlag)
        {
            if(!pP->GetNote()->IsTieBeg())
                DeleteEvent(pP);
            DeleteEventFlag = 0;
        }
    }///end of while(pEM)
    if((rV == 0) && !m_SongDone && !m_StopFlag)
	{
		rV = ProcessEvent();
	}
	if (m_StopFlag || m_SongDone)
		rV = PendingEvents();
    return rV;
}

int CEventQueue::ProcessEvent(void)
{
    int rV = 1;
    CEventMidi *pEM=0;
    CMsEvent *pEv = m_pHead;
    ObjectTypes Obj;
    int loop = 1;
	int EventsAdded = 0;
	int flag=0,Velocity;
	double MsPerTick;

    while(loop)
    {
        while(pEv && pEv->GetHead() == 0) ///no song objects this event
            pEv = pEv->GetNext();
        if(pEv)
        {
			m_pStaffView->PostMessageA(WM_STAFF_DISP_EVENT, pEv->m_Index, STAFF_DISP_EVENT_NEXT);
            Obj.pObj = pEv->GetHead();
            while(Obj.pObj)
            {
                switch(Obj.pObj->GetType())
                {
                    case MSOBJ_BAR:
						m_pWnd->SendMessage(WM_PLAYER,PLAYER_SONG_BAR,Obj.pBar->GetBarID());
                        break;
                    case MSOBJ_END:
                        rV = 0;
						m_pStaffView->PostMessageA(WM_STAFF_DISP_EVENT, 0, STAFF_DISP_EVENT_END);
                        break;
                    case MSOBJ_KEYSIG:
						m_pMsKeySig = Obj.pKey;
                         break;
                    case MSOBJ_LOUDNESS:
						m_Loudness = Obj.pLoud->GetLoudness();
						if(m_Loudness < 80) m_Loudness = 80;
                        break;
                    case MSOBJ_NOTE:
						///-------------------------------------------------
						// This gets complicated.  If the note has a TieEnd
						// flag set, we do NOT want to create a new EventMidi
						// object, rather, we just want to keep using
						// the one that is already there
						//--------------------------------------------------
						pEM = this->FindEvent(Obj.pNote);
						if(pEM == 0)
						{
							pEM = new CEventMidi;
							flag = 1;
						}
						else
							flag = 0;
                        pEM->SetNote(Obj.pNote);
						if(Obj.pNote->IsAccent())
							Velocity = 127;
						else
							Velocity = m_Loudness;
						pEM->SetVelocity(Velocity);
						if(this->CheckChan(Obj.pNote->GetTrack(),pEM->GetChannel()) )
						{
							ChangePatch(pEM->GetChannel(),Obj.pNote->m_pMidiInfo->GetPatch(Obj.pNote->GetTrack()));
						}
                        pEM->SetTick(Obj.pNote->GetDuration(),/*m_Tempo*/1);
						if(Obj.pNote->IsTieend())
						{
							EventsAdded++;
						}
						else
						{
							if(!pEM->GetNote()->IsRest() )
							{
								pEM->GetNote()->NoteOn(Velocity,m_pMsKeySig);
							}
							++EventsAdded;
						}
						if(!flag)
							pEM = 0;
                        break;
                    case MSOBJ_REPEATEND:
						this->m_pWnd->SendMessage(WM_PLAYER,PLAYER_SONG_REPEAT_END,0);
						{
							CMsRepeatStackObject *pRSO;
							pRSO = this->GetTopOfRepeatStack();
							pRSO->m_Count--;
							if(pRSO->m_Count)	//still repeating
							{
								pEv = pRSO->m_pStart;
							}
							else
							{
								///not repeating any more
								this->PopRepeatStack();
							}
						}
                       break;
                    case MSOBJ_REPEATSTART:
						this->m_pWnd->SendMessage(WM_PLAYER,PLAYER_SONG_REPEAT,Obj.pRepS->GetCount());
 						{
							CMsRepeatStackObject *pRSO = new CMsRepeatStackObject;
							pRSO->m_Count = Obj.pRepS->GetCount();
							pRSO->m_pStart = pEv;
							this->PushRepeatStack(pRSO);
						}
                        break;
                    case MSOBJ_TEMPO:
						MsPerTick = 60000.0 /double(Obj.pTemp->m_Tempo);
						MsPerTick /= 48.0;
						//Tempo is in quarter notes per Minute
						//We have a 1ms timer.
						//so, number of ms in a Minute:which is 60,000
						//So for 120 beats/min, a quarter note is 500mS
						//Arbitrarily pick 32nd note as shortest makes it
						// 62.5 mS.  The shortest note has a duration of
						// 4, so the basic time count is 15.6mSec.
						//-------------------------------------------------
						if(MsPerTick < 1.0) MsPerTick = 1.0;
						m_Tempo = (int)MsPerTick;
						m_pWnd->SendMessage(WM_PLAYER,PLAYER_SONG_TEMPO,m_Tempo);
                        break;
                }
				if(pEM)
				{
					AddEvent(pEM);
					EventsAdded++;
					pEM = 0;
					rV =0;
				}
				Obj.pObj = Obj.pObj->GetNext();
				if(!Obj.pObj)
					this->m_pWnd->SendMessage(WM_PLAYER,PLAYER_SONG_EVENTEND,pEv->m_Index);
			}	///end of while(Obj.pObj)
        }	///end of if(pEv)
        else
        {
            loop = 0;   ///Ran out of events
			m_SongDone = 1;
        }
		if(pEv)
			pEv = pEv->GetNext();
		if(EventsAdded)
		{
			loop = 0;
			m_pHead = pEv;	//Set current song position
		}
    }	///end of while(loop);
    return m_TotalEvents && !m_SongDone;
}

void CEventQueue::Print(FILE *pO)
{
//	fprintf(pO,"Total Events:%d\n",m_TotalEvents);
	int i;
	CEventMidi *pEM;
	pEM = this->m_EventHead;
	for(i=0;i<8;++i)
	{
		if(pEM)
		{
//			pEM->Print(stdout);
			pEM = pEM->GetNext();
		}
	}
}

CEventMidi * CEventQueue::FindEvent(CMsNote *pNote)
{
	CEventMidi *pEM = this->m_EventHead;
	int loop = 1;
	while(pEM && loop)
	{
		if((pNote->GetPitch() == pEM->GetNote()->GetPitch()) && (pNote->m_pMidiInfo->GetChannel(pNote->GetTrack()) == pEM->GetNote()->m_pMidiInfo->GetChannel(pEM->GetNote()->GetTrack())))
			loop = 0;
		if(loop)
			pEM = pEM->GetNext();
	}
	return pEM;
}


CMsRepeatStackObject * CEventQueue::PopRepeatStack()
{
	CMsRepeatStackObject *pSO;

	pSO = m_pRepeatStack;
	m_pRepeatStack = m_pRepeatStack->m_pNext;
	return pSO;
}

CMsRepeatStackObject * CEventQueue::GetTopOfRepeatStack()
{
	CMsRepeatStackObject *pSO;

	pSO = m_pRepeatStack;
	return pSO;

}

void CEventQueue::PushRepeatStack(CMsRepeatStackObject *pEv)
{
	pEv->m_pNext = m_pRepeatStack;
	m_pRepeatStack = pEv;
}

bool CEventQueue::CheckChan(int track, int chan)
{
	bool rv = true;
	if(this->m_Patches[chan] != track)
		m_Patches[chan] = track;
	else
		rv = false;
	return rv;
}

void CEventQueue::MidiClock()
{
	DWMSG dwMsg;
	dwMsg.mC[2] = 0;
	dwMsg.mC[1] = 0;
	dwMsg.mC[0] = (char)MIDI_CLOCK;
}

void CEventQueue::MidiStart()
{
	DWMSG dwMsg;
	dwMsg.mC[2] = 0;
	dwMsg.mC[1] = 0;
	dwMsg.mC[0] = (char)MIDI_START;
}

void CEventQueue::MidiStop()
{
	DWMSG dwMsg;
	dwMsg.mC[2] = 0;
	dwMsg.mC[1] = 0;
	dwMsg.mC[0] = (char)MIDI_STOP;
}

void CEventQueue::MidiContinue()
{
	DWMSG dwMsg;
	dwMsg.mC[2] = 0;
	dwMsg.mC[1] = 0;
	dwMsg.mC[0] = (char)MIDI_CONTINUE;
}

void CEventQueue::ChangePatch(int chan, int patch)
{
	DWMSG dwMsg;

	dwMsg.mC[2] = 0;
	dwMsg.mC[1] = (char)patch;
	dwMsg.mC[0] = (char)(MIDI_PGMCHANGE | (chan & 0x0f));
	midiOutShortMsg(m_hMidiOut,dwMsg.m_dwMsg);
}

