#ifndef CEVENTQUEUE_H
#define CEVENTQUEUE_H

#include "EventMidi.h"
#include "MsEvent.h"
#include "MidiInfo.h"
#include "MMsystem.h"
#include "MsRepeatStackObject.h"
#include "MsKeySignature.h"	// Added by ClassView

class CEventQueue
{
	HMIDIOUT m_hMidiOut;
	int m_TotalEvents;
    CEventMidi *m_EventHead;
    CEventMidi *m_EventTail;
    CMsEvent *m_pHead;	///current song position
    CMidiInfo *m_pMidiInfo;
	int m_Loudness;
	CMsRepeatStackObject *m_pRepeatStack;
	char m_Patches[16];
	int m_Tempo;
	int m_SongDone;
	int m_MidiClockFlag;
	int m_TempoCount;
	CWnd *m_pWnd;	//parent window
	CWnd *m_pStaffView;	//staff view window
public:
	int GetTotalEvents(void) { return m_TotalEvents; }
	inline void SetStaffView(CWnd *pW) { m_pStaffView = pW;}
	inline CWnd *GetStaffView(void) { return m_pStaffView; }
	inline void SetTempo(int t) { m_Tempo = t; }
	inline int GetTempo(void) { return m_Tempo; }
	inline int DecrTempoCount(void) { return --m_TempoCount; }
	inline int GetTempoCount(void) { return m_TempoCount; }
	inline void SetTempoCount(int c) { m_TempoCount = c; }
	CMsKeySignature *m_pMsKeySig;
	CMsRepeatStackObject * m_RepeatStackTop;
public:
	int m_StopFlag;
    CEventQueue();
    virtual ~CEventQueue();
	//-------------------------------
	// Midi Message Functions
	//-------------------------------
	void ChangePatch(int chan,int patch);
	void MidiContinue(void);
	void MidiStop(void);
	void MidiStart(void);
	void MidiClock(void);
	//-------------------------------
	// CEventMidi management functions
	//-------------------------------
    void AddEvent(CEventMidi *pEM);
    void DeleteEvent(CEventMidi *pEM);
	CEventMidi * FindEvent(CMsNote *pNote);
	//--------------------------------
	// Song playing functions
	//---------------------------------
    int Ticker(void);
    int Shortest(void);
    int Longest(void);
    void SetSongBegin(CMsEvent *pEv){m_pHead = pEv;}
    void Start(void);
	int Stop(void);
    int ProcessEvent(void);
	//-----------------------------------
	// print object to file for debug
	//-----------------------------------
	void Print(FILE *pO);
	//-----------------------------------
	// Function to manage attributes
	//-----------------------------------
	void SetWindow(CWnd *pW){m_pWnd = pW;}
	CWnd *GetWindow(void) { return m_pWnd; }
	void SetMidiOut(HMIDIOUT mo){m_hMidiOut = mo;}
	void SetMidiInfo(CMidiInfo *pMI){m_pMidiInfo = pMI;}
	bool CheckChan(int track, int chan);
	//------------------------------------------
	// Repeat Stack functions
	//------------------------------------------
	void PushRepeatStack(CMsRepeatStackObject *pEv);
	CMsRepeatStackObject * GetTopOfRepeatStack(void);
	CMsRepeatStackObject * PopRepeatStack(void);

	// Checks to see if there are any pending events
	int PendingEvents();
};


#endif // CEVENTQUEUE_H
