 // MSPlayerDlg.cpp : implementation file
//
//#define _CRT_SECURE_NO_WARNINGS

#include "stdafx.h"
#include "MusicStudioPlayer.h"
#include "MSPlayerDlg.h"
#include "MidiInfo.h"
#include "MidiConfig.h"
#include "SelectMidi.h"
#include "MsLoudness.h"
#include "MsTempo.h"
#include "MsTimeSig.h"
#include "MsKeySignature.h"
#include "ParamDlg.h"
#include "SelDurDlg.h"
#include "SelectorDlg.h"
#include "WindowMessages.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//***********************************************
// MMTimer Callback Function
//***********************************************

static void CALLBACK MStimer(UINT uID, UINT uMsg,DWORD dwUser,DWORD dw1,DWORD dw2)
{
	//----------------------------------------------
	//	MStimer
	//		This is the callback function for the
	// Windows Multi Media Timer.  This timer is 
	// Different from a regular Windows timer in that
	// it has a much higher priority than a regular
	// timer.  It is sort of like running code at the
	// interrupt level.  Care must be taken not to
	// call forbidden system functions at this level.
	//
	// parameters:
	//	uID.....ID returned when timer was set up
	//	uMsg....Don't fucking use this variable
	//	dwUser..User suppled parameter
	//	dw1.....DO NOT USE
	// dw2..... DO NOT USE
	//-----------------------------------------------
	CMsSong *pSong = (CMsSong *)dwUser;
	CEventQueue *pEQ = pSong->GetEventQueue();
	if(pEQ->GetTempoCount() == 0)
	{
		//------------------------------
		// if we have reached the end
		// of the tempo count, first
		// reset the counterm tgeb call
		// the ticker function to process
		// events.
		//-------------------------------
		pEQ->SetTempoCount(pEQ->GetTempo() - 1);
		if(!pEQ->Ticker())
		{
			//--------------------------------
			// if Ticker() returns 0, the song
			// is done playing, being either at
			// the end, or because the user
			// stopped it.
			//-------------------------------
			pEQ->Stop();
			pSong->m_nIsPlaying = 0;
			pEQ->GetWindow()->PostMessageA(WM_PLAYER,PLAYER_SONG_END,0);
			pEQ->GetStaffView()->PostMessageA(WM_STAFF_DISP_EVENT, 0, STAFF_DISP_EVENT_END);
			pSong->m_pWndParent->PostMessageA(WM_PLAYER, PLAYER_SONG_PLAYBUTTON, 1);
		}
	}
	else
		pEQ->DecrTempoCount();	//decrement tempo count
}

//-------------------------------------------------
// MidiInProc
//-------------------------------------------------

void CALLBACK MidiInProc(HMIDIIN hMidiIn,UINT wMsg,DWORD dwInst,DWORD dwP1,DWORD dwP2)
{
	CMSPlayerDlg *pDlg = (CMSPlayerDlg *)dwInst;
	switch(wMsg)
	{
		case MIM_CLOSE:
			break;
		case MIM_DATA:
			pDlg->m_pStaffView->PostMessageA(WM_SHORTMIDIMSG, dwP1, dwP2);
			pDlg->PostMessage(WM_MIDIIN,dwP1,dwP2);
			break;
		case MIM_ERROR:
			break;
		case MIM_LONGDATA:
			{
				MIDIHDR *pMHdr = (MIDIHDR *)dwP1;
				pDlg->PostMessage(WM_MIDIIN_LONG,dwP1,dwP2);
			}
			break;
		case MIM_LONGERROR:
			break;
		case MIM_MOREDATA:
			break;
		case MIM_OPEN:
			break;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMSPlayerDlg dialog

int InstBMidLUT[15] = {
	IDB_INST_1,
	IDB_INST_2,
	IDB_INST_3,
	IDB_INST_4,
	IDB_INST_5,
	IDB_INST_6,
	IDB_INST_7,
	IDB_INST_8,
	IDB_INST_9,
	IDB_INST_10,
	IDB_INST_11,
	IDB_INST_12,
	IDB_INST_13,
	IDB_INST_14,
	IDB_INST_15
};

int AccedentLUT[4] = {
	ACCIDENT_INKEY,
	ACCIDENT_SHARP,
	ACCIDNET_FLAT,
	ACCIDENT_NATURAL
};

int NoteDurLut[6] ={
	WHOLE_NOTE,
	HALF_NOTE,
	QUARTER_NOTE,
	EIGTH_NOTE,
	SIXTEENTH_NOTE,
	THIRTYSEC_NOTE
};

CMSPlayerDlg::CMSPlayerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMSPlayerDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMSPlayerDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_pSong = 0;
	m_MidiOutFlag = 0;
	m_MidiInFlag = 0;
	m_NoteAccent = 0;
	m_NoteTriplet = 0;
	m_NoteDotted = 0;
	m_NoteDur = 0;
	m_NoteTrack = 0;
	m_RestDur = 0;
	m_NoteRest = 0;
	m_NoteKey = 0;
	m_KeySig = 0;
	m_TimeSig = 4;	///default time sig is 4/4
	m_hMidiIn = 0;
	m_hMidiOut = 0;
}

void CMSPlayerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMSPlayerDlg)
	DDX_Control(pDX, IDC_STATIC_STATUS, m_Static_DisplayStatus);
	DDX_Control(pDX, IDC_STATIC_TEMPO, m_Static_Tempo);
	DDX_Control(pDX, IDC_SCROLLBAR_TEMPO, m_ScrollBar_Tempo);
	DDX_Control(pDX, IDC_STATIC_LOUDNESS, m_Static_Loudness);
	DDX_Control(pDX, IDC_SCROLLBAR_LOUDNESS, m_ScrollBar_Loudness);
	DDX_Control(pDX, IDC_COMBO_KEYSIG, m_Combo_KeySig);
	DDX_Control(pDX, IDC_COMBO_TIMESIG, m_Combo_TimeSig);
	DDX_Control(pDX, IDC_COMBO_INSTRUMENT, m_Combo_Instrument);
	DDX_Control(pDX, IDC_COMBO_TIEBAR, m_Combo_TieBar);
	DDX_Control(pDX, IDC_COMBO_REST, m_Combo_Rests);
	//  DDX_Control(pDX, IDC_COMBO_DECORATIONS, m_Combo_Decorations);
	DDX_Control(pDX, IDC_COMBO_BLOCKOPS, m_Combo_BlockOps);
	DDX_Control(pDX, IDC_COMBO_ACCIDENT, m_Combo_Accident);
	DDX_Control(pDX, IDC_COMBO_NOTEMENU, m_Combo_NoteType);
	DDX_Control(pDX, IDC_STATIC_MESSAGE, m_Static_Message);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDC_BUTTON_PLAY, m_Button_Play);
	DDX_Control(pDX, IDC_COMBO_DECORATIONS, m_Combo_Decorations);
}

BEGIN_MESSAGE_MAP(CMSPlayerDlg, CDialog)
	//{{AFX_MSG_MAP(CMSPlayerDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_COMMAND(IDM_FILE_OPEN, OnFileOpen)
	ON_COMMAND(IDM_FMENU_FILE_EXIT, OnFmenuFileExit)
	ON_COMMAND(IDM_MENU_FILE_PLAY, OnMenuFilePlay)
	ON_COMMAND(IDM_MENU_SETUP_MIDI, OnMenuSetupMidi)
	ON_COMMAND(IDM_MENU_SETUP_SELMIDI, OnMenuSetupSelmidi)
	ON_BN_CLICKED(IDC_BUTTON_PLAY, OnButtonPlay)
	ON_CBN_SELCHANGE(IDC_COMBO_ACCIDENT, OnSelchangeComboAccident)
	ON_CBN_SELCHANGE(IDC_COMBO_BLOCKOPS, OnSelchangeComboBlockops)
	ON_CBN_SELCHANGE(IDC_COMBO_DECORATIONS, OnSelchangeComboDecorations)
	ON_CBN_SELCHANGE(IDC_COMBO_INSTRUMENT, OnSelchangeComboInstrument)
	ON_CBN_SELCHANGE(IDC_COMBO_NOTEMENU, OnSelchangeComboNotemenu)
	ON_CBN_SELCHANGE(IDC_COMBO_REST, OnSelchangeComboRest)
	ON_CBN_SELCHANGE(IDC_COMBO_TIEBAR, OnSelchangeComboTiebar)
	ON_COMMAND(IDM_FILE_NEW, OnFileNew)
	ON_COMMAND(IDM_FILE_SAVEAS, OnFileSaveas)
	ON_COMMAND(IDM_FILE_SAVE, OnFileSave)
	ON_CBN_SELCHANGE(IDC_COMBO_KEYSIG, OnSelchangeComboKeysig)
	ON_CBN_SELCHANGE(IDC_COMBO_TIMESIG, OnSelchangeComboTimesig)
	ON_BN_CLICKED(IDC_BUTTON_STOP, OnButtonStop)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_PLAYER,OnMsgPlay)
	ON_MESSAGE(WM_PGMCHANGE,OnPgmChange)
	ON_MESSAGE(WM_MIDIIN,OnMidiInMsg)
	ON_MESSAGE(WM_MIDIIN_LONG,OnMidiInLongMsg)
	ON_COMMAND(ID_FILE_DUMP, &CMSPlayerDlg::OnFileDump)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMSPlayerDlg message handlers

BOOL CMSPlayerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	//put a CView into the dialog
	CRect rectView;
	//----------------------------------------
	// First, we need to get a window object
	// that points to the object in the
	// dialog where we are going to put the
	// the view.
	//----------------------------------------
	CWnd *pWnd = GetDlgItem(IDC_STATIC_STAFF);
	//----------------------------------------
	// we need to get the rectangle that
	// describes the dimensions of the
	// view
	//----------------------------------------
	pWnd->GetWindowRect(&rectView);
	ScreenToClient(&rectView);
	//----------------------------------------
	// We now create our view class
	//----------------------------------------
	CRuntimeClass *pClass = RUNTIME_CLASS(CStaffView);
	m_pStaffView = (CStaffView *)pClass->CreateObject();
	m_pStaffView->Create(NULL, NULL, WS_CHILD | WS_VISIBLE, rectView, this, IDC_STATIC_STAFF);
	m_pStaffView->ModifyStyleEx(0, WS_EX_CLIENTEDGE);
	//----------------------------------------
	// We send a message to all children to do
	// their initial update.
	//
	// so, where exactly does WM_INITIALUPDATE
	// come from?  This was a tough question.
	// In my VC++ 6.0 MSDN docs, I was unable
	// to find this.  And yet, the demo
	// program I had compiled OK.  What you
	// need to do is make sure you put
	//			#include <afxpriv.h>
	// into the stdafx.h file.  I suspect
	// you are not really supposed to do that,
	// but why not?
	//----------------------------------------
	SendMessageToDescendants(WM_INITIALUPDATE, 0, 0, FALSE);
	//----------------------------------------
	// destroy this temporary window
	//----------------------------------------
	pWnd->DestroyWindow();
	
	//Rest of initialization
	int i;
	//--------------------------------
	// Select midi in and out
	//--------------------------------
	OnMenuSetupSelmidi();
	//---------------------------------
	// Import bitmaps for Instruments
	//---------------------------------
	CBitmap *pBM = &m_BmInst1;
	for (i = 0; i<15; ++i, ++pBM)
	{
		pBM->LoadBitmapA(InstBMidLUT[i]);
		//		pBM = m_pSong->GetMidiInfo()->GetInstColor(i+1)->GetBitmap();
		m_Combo_Instrument.AddBitmap(pBM, "");
	}
	m_Combo_Instrument.SetCurSel(0);
	//------------------------------------
	// Create initial song file
	//-------------------------------------
	OnFileNew();
	//------------------------------------
	// Initialize song buffer
	//------------------------------------
	m_pRawSongData = new char[65368];
	//------------------------------------
	// Setup debug display
	//-----------------------------------
	m_Static_Message.InitStringFifo(8);
	//-----------------------------------
	// Setup Staff View Class
	//-----------------------------------
	// Tell StaffView where program change is
	//-------------------------------------
	m_pStaffView->m_pPgmChng = this;
	// Tell StaffView where Status display is
	m_pStaffView->m_pStatus = &this->m_Static_DisplayStatus;
	// Tell staff view where default keysignature is
	m_pStaffView->m_pKeySig = &this->m_MsKeySig;
	// Tell Staff view here debug display is
	m_pStaffView->m_pDisp = &m_Static_Message;
	// have staff view calculate the maximum
	// number of events in one view.
	m_pStaffView->CalcMaxEvents();
	// Initialize staff
	m_pStaffView->SendMessage(WM_INIT_STAFF);
	m_MaxEvents = m_pStaffView->m_MaxEvents;
	//---------------------------------------
	// Initailize keysignature combo box
	//----------------------------------------
	for(i=0;i<15;++i)
		this->m_Combo_KeySig.AddString(KeySigStringTab[i+1]);
	m_Combo_KeySig.SetCurSel(m_KeySig);
	this->m_MsKeySig.SetKey(m_KeySig);
	//----------------------------------------
	// Initialize time signature combo box
	//-----------------------------------------
	m_BmTime22.LoadBitmap(IDB_TIMESIG_2_2);
	this->m_Combo_TimeSig.AddBitmap(&m_BmTime22,"");
	m_BmTime32.LoadBitmap(IDB_TIMESIG_3_2);
	this->m_Combo_TimeSig.AddBitmap(&m_BmTime32, "");
	m_BmTime24.LoadBitmap(IDB_TIMESIG_2_4);
	this->m_Combo_TimeSig.AddBitmap(&m_BmTime24, "");
	m_BmTime34.LoadBitmap(IDB_TIMESIG_3_4);
	this->m_Combo_TimeSig.AddBitmap(&m_BmTime34, "");
	m_BmTime44.LoadBitmap(IDB_TIMESIG_4_4);
	this->m_Combo_TimeSig.AddBitmap(&m_BmTime44, "");
	m_BmTime54.LoadBitmap(IDB_TIMESIG_5_4);
	this->m_Combo_TimeSig.AddBitmap(&m_BmTime54, "");
	m_BmTime68.LoadBitmap(IDB_TIMESIG_6_8);
	this->m_Combo_TimeSig.AddBitmap(&m_BmTime68, "");
	m_Combo_TimeSig.SetCurSel(m_TimeSig);
	//------------------------------------------
	// Initialize note duration combox box
	//------------------------------------------
	m_BmWhole.LoadBitmap(IDB_NOTE_WHOLE);
	m_Combo_NoteType.AddBitmap(&m_BmWhole, "");
	m_BmHalf.LoadBitmap(IDB_NOTE_HALF);
	m_Combo_NoteType.AddBitmap(&m_BmHalf, "");
	m_BmQuarter.LoadBitmap(IDB_NOTE_QUARTER);
	m_Combo_NoteType.AddBitmap(&m_BmQuarter, "");
	m_BmEigth.LoadBitmap(IDB_NOTE_EIGTH);
	m_Combo_NoteType.AddBitmap(&m_BmEigth, "");
	m_BmSixteenth.LoadBitmap(IDB_NOTE_SIXTEENTH);
	m_Combo_NoteType.AddBitmap(&m_BmSixteenth, "");
	m_BmThirtysec.LoadBitmap(IDB_NOTE_THIRTYSECOND);
	m_Combo_NoteType.AddBitmap(&m_BmThirtysec, "");
	m_Combo_NoteType.SetCurSel(2);
	m_NoteDur = NoteDurLut[m_Combo_NoteType.GetCurSel()];
	//---------------------------------------
	// Initialize rest duration combo box
	//---------------------------------------
	m_BmWholeRest.LoadBitmap(IDB_WHOLEREST);
	m_Combo_Rests.AddBitmap(&m_BmWholeRest, "");
	m_BmHalfRest.LoadBitmap(IDB_HALFREST);
	m_Combo_Rests.AddBitmap(&m_BmHalfRest, "");
	m_BmQuarterRest.LoadBitmap(IDB_BITMAP_QUARTERREST);
	m_Combo_Rests.AddBitmap(&m_BmQuarterRest, "");
	m_BmEigthRest.LoadBitmap(IDB_EIGHTHREST);
	m_Combo_Rests.AddBitmap(&m_BmEigthRest, "");
	m_BmSixteenthRest.LoadBitmap(IDB_SIXTEENTHREST);
	m_Combo_Rests.AddBitmap(&m_BmSixteenthRest, "");
	m_BmThirtySecRest.LoadBitmap(IDB_THIRTYSEC_REST);
	m_Combo_Rests.AddBitmap(&m_BmThirtySecRest, "");
	m_Combo_Rests.SetCurSel(2);
	m_RestDur = NoteDurLut[m_Combo_Rests.GetCurSel()];
	//-------------------------------------------
	// Initialize decorations combo box
	//--------------------------------------------
	m_BmAccent.LoadBitmap(IDB_ACCENT);
	m_Combo_Decorations.AddBitmap(&m_BmAccent,"");
	m_BmNone.LoadBitmapA(IDB_DECOR_NONE);
	m_Combo_Decorations.AddBitmap(&m_BmNone, "None");
	m_BmDot.LoadBitmap(IDB_DOT);
	m_Combo_Decorations.AddBitmap(&m_BmDot,"");
	m_BmTripplet.LoadBitmap(IDB_TRIPLET);
	m_Combo_Decorations.AddBitmap(&m_BmTripplet,"");
	m_Combo_Decorations.SetCurSel(1);
	m_Combo_Decorations.EnableCheckbox(0, 1);
	//-----------------------------------------
	// Initialize Block Operations combo box
	//------------------------------------------
	m_BmBlockCopy.LoadBitmap(IDB_BLOCK_COPY);
	m_Combo_BlockOps.AddBitmap(&m_BmBlockCopy, "");
	m_BmBlockMove.LoadBitmap(IDB_BLOCK_MOVE);
	m_Combo_BlockOps.AddBitmap(&m_BmBlockMove, "");
	m_BmBlockChngInst.LoadBitmap(IDB_BLOCK_CHANGEINSTRUMENT);
	m_Combo_BlockOps.AddBitmap(&m_BmBlockChngInst, "");
	m_BmBlockChngDur.LoadBitmap(IDB_BLOCK_CHANGEDUR);
	m_Combo_BlockOps.AddBitmap(&m_BmBlockChngDur, "");
	m_BlockIncrDur.LoadBitmap(IDB_BLOCK_INCREASE);
	m_Combo_BlockOps.AddBitmap(&m_BlockIncrDur, "");
	m_BlockDecDur.LoadBitmap(IDB_BLOCK_DECREASE);
	m_Combo_BlockOps.AddBitmap(&m_BlockDecDur, "");
	m_BlockTransUp.LoadBitmap(IDB_BLOCKUP);
	m_Combo_BlockOps.AddBitmap(&m_BlockTransUp, "");
	m_BlockTransDown.LoadBitmap(IDB_BLOCKDOWN);
	m_Combo_BlockOps.AddBitmap(&m_BlockTransDown, "");
	m_BlockRepeat.LoadBitmap(IDB_BLOCK_REPEAT);
	m_Combo_BlockOps.AddBitmap(&m_BlockRepeat, "");
	m_BlockInsert.LoadBitmap(IDB_BLOCK_INSERT);
	m_Combo_BlockOps.AddBitmap(&m_BlockInsert, "");
	m_BlockMidiInput.LoadBitmap(IDB_BLOCK_MIDIINPUT);
	m_Combo_BlockOps.AddBitmap(&m_BlockMidiInput, "");
	m_Combo_BlockOps.SetCurSel(0);
	//--------------------------------------
	// Initialize accidental combo box
	//---------------------------------------
	m_BmInKey.LoadBitmap(IDB_INKEY);
	m_Combo_Accident.AddBitmap(&m_BmInKey, "");
	m_BmSharp.LoadBitmap(IDB_SHARP);
	m_Combo_Accident.AddBitmap(&m_BmSharp, "");
	m_BmFlat.LoadBitmap(IDB_FLAT);
	m_Combo_Accident.AddBitmap(&m_BmFlat, "");
	m_BmNatural.LoadBitmap(IDB_NATURAL);
	m_Combo_Accident.AddBitmap(&m_BmNatural, "");
	m_Combo_Accident.SetCurSel(0);
	m_NoteKey = AccedentLUT[m_Combo_Accident.GetCurSel()];
	//----------------------------------------
	// Initialize Misc Stuff Combo box
	//-----------------------------------------
	m_BmTie.LoadBitmap(IDB_MEASUREBAR);
	m_Combo_TieBar.AddBitmap(&m_BmTie, "");
	m_BmBar.LoadBitmap(IDB_NOTETIE);
	m_Combo_TieBar.AddBitmap(&m_BmBar, "");
	m_BmLoudness.LoadBitmap(IDB_LOUDNESS);
	m_Combo_TieBar.AddBitmap(&m_BmLoudness, "");
	m_BmTempo.LoadBitmap(IDB_TEMPO);
	m_Combo_TieBar.AddBitmap(&m_BmTempo, "");
	m_BmInstChange.LoadBitmap(IDB_INST_CHANGE);
	m_Combo_TieBar.AddBitmap(&m_BmInstChange, "");
	m_Combo_TieBar.SetCurSel(1);
	//---------------------------------------------


	m_ScrollBar_Loudness.SetScrollRange(0,127);
	m_ScrollBar_Loudness.SetDisplayText(&m_Static_Loudness);
	m_ScrollBar_Loudness.SetScrollPos(100);

	m_ScrollBar_Tempo.SetScrollRange(56,200);
	m_ScrollBar_Tempo.SetDisplayText(&m_Static_Tempo);
	m_ScrollBar_Tempo.SetScrollPos(160);
	

	OnSelchangeComboInstrument();
	m_pLog = ((CMSPlayerApp *)AfxGetApp())->GetLogFile();
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CMSPlayerDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CMSPlayerDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CMSPlayerDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CMSPlayerDlg::OnFileOpen() 
{
	// TODO: Add your command handler code here
	CFileDialog Dlg(true);

	if(Dlg.DoModal() == IDOK)
	{
		CString name = Dlg.GetPathName();
		FILE *pIn = fopen(LPCTSTR(name),"rb");
		if(pIn)
		{
			fread(this->m_pRawSongData,1,64000,pIn);
			fclose(pIn);
			SetWindowText(LPCTSTR(name));
			if(m_pSong)
			{
				m_pStaffView->m_pSong = 0;
				delete m_pSong;
			}
			m_pSong = new CMsSong;
			m_pSong->m_hMidiOut = m_hMidiOut;
			m_pSong->CreateMidiInfo(&m_pRawSongData[MS_MIDIINSTURMENTS_OFFSET]);
			m_pSong->Parse(&m_pRawSongData[MS_SONGDATA_OFFSET]);
			m_pSong->m_pWndParent = this;
			m_pStaffView->m_pSong = m_pSong;
			m_pStaffView->Invalidate();
		}
		else
		{
			MessageBox(LPCTSTR(name),"Could Not Open");
		}
	}
	
}

void CMSPlayerDlg::OnFmenuFileExit() 
{
	// TODO: Add your command handler code here
	CDialog::OnOK();
}

void CMSPlayerDlg::OnMenuFilePlay() 
{
	// TODO: Add your command handler code here
	OnButtonPlay();
}

void CMSPlayerDlg::OnMenuSetupMidi() 
{
	// TODO: Add your command handler code here
	CMidiConfig Dlg;

	Dlg.m_pSong = this->m_pSong;
	Dlg.m_pWndPgmChange = this;
	Dlg.DoModal();
}

void CMSPlayerDlg::OnMenuSetupSelmidi() 
{
	// TODO: Add your command handler code here
	CSelectMidi Dlg;
	UINT MidiOutIndex,MidiInIndex;

	if(Dlg.DoModal() == IDOK)
	{
		MidiOutIndex = Dlg.MidiOutIndex;
		MidiInIndex = Dlg.MidiInIndex;
		if(m_hMidiIn)
			midiInClose(m_hMidiIn);
		if(midiInOpen(&m_hMidiIn,MidiInIndex,(DWORD)MidiInProc,(DWORD)this,CALLBACK_FUNCTION) == MMSYSERR_NOERROR)
		{
			m_MidiInFlag = 1;
			//----------------------------
			// create a pair of midi in
			// buffers
			//-----------------------------
			midiInPrepareHeader(m_hMidiIn,&this->m_MidiIn1,sizeof(MIDIHDR));
			midiInPrepareHeader(m_hMidiIn,&this->m_MidiIn2,sizeof(MIDIHDR));
			m_MidiIn1.lpData = new char[4096];
			m_MidiIn2.lpData = new char[4096];
			m_MidiIn1.dwBufferLength = 4096;
			m_MidiIn2.dwBufferLength = 4096;
			midiInStart(m_hMidiIn);
		}
		else
		{
			MessageBox("Could Not Open Midi In","ERROR");
		}
		if(midiOutOpen(&m_hMidiOut,MidiOutIndex,0,0,0) == MMSYSERR_NOERROR)
		{
			m_MidiOutFlag = 1;
		}
		else
		{
			MessageBox("Could Not Open Midi Out","ERROR");
		}
	}
}

void CMSPlayerDlg::OnOK() 
{
	// TODO: Add extra validation here
	
	CDialog::OnOK();
}

void CMSPlayerDlg::OnButtonPlay() 
{
	// TODO: Add your control notification handler code here
	if(m_MidiOutFlag)
	{
		m_Button_Play.EnableWindow(0);
		if(m_pSong->Play(m_hMidiOut,this,m_pStaffView))
			m_TimerID = timeSetEvent(1,1,MStimer,(unsigned long)m_pSong,TIME_PERIODIC);
	}
	else
		MessageBox("Open up a Midi Port","ERROR");
}

void CMSPlayerDlg::OnButtonStop() 
{
	// TODO: Add your control notification handler code here
	m_pSong->Stop();
}

LRESULT CMSPlayerDlg::OnMsgPlay(WPARAM wp, LPARAM lp)
{
	char s[128];
	char *pMS;

	switch(wp)	//decode submessage
	{
		case PLAYER_SONG_BAR:
			sprintf_s(s,128,"BAR=%d",lp);
			if (m_pLog) fprintf(m_pLog, "%s\n", s);
			m_Static_Message.SetWindowText(s);
			break;
		case PLAYER_SONG_END:
			timeKillEvent(m_TimerID);
			if (m_pLog) fprintf(m_pLog, "DONE\n");
			m_Static_Message.SetWindowText("DONE");
			break;
		case PLAYER_SONG_ERROR:
			sprintf_s(s,128,"ERROR IN BAR %d",lp);
			m_Static_Message.SetWindowText(s);
			if (m_pLog) fprintf(m_pLog, "%s\n", s);
			break;
		case PLAYER_SONG_REPEAT:	
			sprintf_s(s,128,"REPEAT START %d",lp);
			m_Static_Message.SetWindowText(s);
			break;
		case PLAYER_SONG_REPEAT_END:
			sprintf_s(s,128,"REPEAT END %d",lp);
			m_Static_Message.SetWindowText(s);
			break;
		case PLAYER_SONG_TEMPO:
			sprintf_s(s,128,"Tempo = %d",lp);
			m_Static_Message.SetWindowText(s);
			break;
		case PLAYER_SONG_NOTE:
			sprintf_s(s,128,"Duration = %d",lp);
			m_Static_Message.SetWindowText(s);
			break;
		case PLAYER_SONG_PATCHCHNG:
			sprintf_s(s,128,"Patch Change %d",lp);
			m_Static_Message.SetWindowText(s);
			break;
		case PLAYER_SONG_TICKDEBUG:
			{
				CEventMidi *pEM = (CEventMidi *)lp;
				if(m_pLog) pEM->Print(m_pLog);
			}
			break;
		case PLAYER_SONG_EVENTEND:
			if(m_pLog) fprintf(m_pLog,"Event END----------- %d -------------\n",lp);
			break;
		case PLAYER_SONG_PLAYBUTTON:
			m_Button_Play.EnableWindow(lp);
			break;
		case PLAYER_SONG_NOTEONOFF:
			if ((lp & 0xf0) == 0x80) pMS = "NOTEOFF";
			else pMS = "NOTEON";
			if(m_pLog && m_pSong->m_nIsPlaying) fprintf(m_pLog, "NOTEONOFF: %s:%d-%02x-%02x.....Total Queue Events:%d\n", pMS,lp & 0x0f, (lp >> 8) & 0x0ff, (lp >> 16) & 0x0ff, m_pSong->GetQueueEventTotal());
			break;
	}
	return 0;
}

void CMSPlayerDlg::OnSelchangeComboAccident() 
{
	// TODO: Add your control notification handler code here
	m_NoteKey = AccedentLUT[m_Combo_Accident.GetCurSel()];
	m_pStaffView->SendMessage(WM_DRAWMODE,DRAW_NOTE,CreateNoteMessageWord());	
}

void CMSPlayerDlg::OnSelchangeComboBlockops() 
{
	// TODO: Add your control notification handler code here
	int mode=0,v=0;
	int op = this->m_Combo_BlockOps.GetCurSel();
	switch(op)
	{
		case 0:	//block copy
			mode = DRAW_COPY;
			break;
		case 1:	//block move
			mode = DRAW_MOVE;
			break;
		case 2:	//change instrument
			{
				int InstFrom,InstTo;
				{
					CSelectorDlg Dlg;
					Dlg.m_pBitmaps = &m_BmInst1;
					Dlg.m_nBitmaps = 15;
					Dlg.m_pCaptionString = "Instrument FROM";
					Dlg.m_nSelection = 0;
//					CSelInstDlg Dlg;
//					Dlg.m_pPromtString = "FROM";
					if (IDOK == Dlg.DoModal())
					{
						InstFrom = Dlg.m_nSelection+1;
						Dlg.m_pBitmaps = &m_BmInst1;
						Dlg.m_pCaptionString = "Instrument TO";
						Dlg.m_nSelection = 0;
						if (IDOK == Dlg.DoModal())
						{
							InstTo = Dlg.m_nSelection+1;
							v = (InstFrom & 0xf) | (InstTo << 4);
							mode = DRAW_INSTCHANGE;
						}
						else
							mode = DRAW_NOP;
					}
					else
						mode = DRAW_NOP;
				}
			}
			break;
		case 3:	//change duration
			{
				int DurFrom,DurTo;
				{
					CSelDurDlg Dlg;
					Dlg.m_nDuration = 12;
					Dlg.m_pPrompt = "FROM";
					Dlg.DoModal();
					DurFrom = Dlg.m_nDuration;
					mode = DRAW_CHANGEDUR;
				}
				{
					CSelDurDlg Dlg;
					Dlg.m_pPrompt = "TO";
					Dlg.m_nDuration = 12;
					Dlg.DoModal();
					DurTo = Dlg.m_nDuration;
					v = (DurFrom & 0x1f) | (DurTo << 5);
					mode = DRAW_CHANGEDUR;
				}
			}
			break;
		case 4:	//Increase duration
			mode = DRAW_INCREASEDUR;
			break;
		case 5:	//Decrease duration
			mode = DRAW_DECREASEDUR;
			break;
		case 6:	//increase pitch up
			mode = DRAW_INCRPITCH;
			break;
		case 7:	//increase pitch down
			mode = DRAW_DECRPITCH;
			break;
		case 8:	//add repeat
			{
				CParamDlg Dlg;
				Dlg.m_pCaption = "Repeat Count";
				Dlg.m_Min = 1;
				Dlg.m_Max = 255;
				Dlg.m_Val = 2;
				Dlg.DoModal();
				v = Dlg.m_Val;
				mode = DRAW_REPEAT;
			}
			break;
		case 9:	//insert blok
			mode = DRAW_INSERTBLOCK;
			break;
		case 10:	//Insert notes via midi
			mode = DRAW_DRAWNOTESVIAMIDI;
			m_NoteDur = NoteDurLut[m_Combo_NoteType.GetCurSel()];
			this->m_NoteRest = 0;
			v = CreateNoteMessageWord();
			break;
	}
	m_pStaffView->SendMessage(WM_DRAWMODE,mode,v);	
}

void CMSPlayerDlg::OnSelchangeComboDecorations() 
{
	int sel;
	sel = m_Combo_Decorations.GetCurSel();
	m_NoteAccent = m_Combo_Decorations.GetCheck(COMBO_DECOR_ACCENT);
	switch(sel)
	{
		case COMBO_DECOR_NONE:
			m_NoteDotted = 0;
			m_NoteTriplet = 0;
			break;
		case COMBO_DECOR_DOT:
			m_NoteDotted = 1;
			m_NoteTriplet = 0;
			break;
		case COMBO_DECOR_TRIPLET:		
			m_NoteDotted = 0;
			m_NoteTriplet = 1;
			break;
	}
	m_pStaffView->SendMessage(WM_DRAWMODE,DRAW_NOTE,CreateNoteMessageWord());	
}

void CMSPlayerDlg::OnSelchangeComboInstrument() 
{
	// TODO: Add your control notification handler code here
	m_NoteTrack = m_Combo_Instrument.GetCurSel()+1;		//Instrument number needs +1
	m_pStaffView->SendMessage(WM_DRAWMODE,DRAW_NOTE,CreateNoteMessageWord());
	this->SendMessage(WM_PGMCHANGE,m_pSong->GetMidiInfo()->GetPatch(m_NoteTrack),m_NoteTrack);
}

//-------------------------------
// Note Combo Box handler
//--------------------------------

void CMSPlayerDlg::OnSelchangeComboNotemenu() 
{
	// TODO: Add your control notification handler code here
	m_NoteDur = NoteDurLut[m_Combo_NoteType.GetCurSel()];
	this->m_NoteRest = 0;
	m_pStaffView->SendMessage(WM_DRAWMODE,DRAW_NOTE,CreateNoteMessageWord());	
}

void CMSPlayerDlg::OnSelchangeComboRest() 
{
	// TODO: Add your control notification handler code here
	m_RestDur = NoteDurLut[m_Combo_Rests.GetCurSel()];
	this->m_NoteRest = 1;
	this->m_NoteDur = m_RestDur;
	m_pStaffView->SendMessage(WM_DRAWMODE,DRAW_NOTE,CreateNoteMessageWord());	
}

void CMSPlayerDlg::OnSelchangeComboTiebar() 
{
	// TODO: Add your control notification handler code here
	int Selection,mode;
	int v=0;

	Selection = this->m_Combo_TieBar.GetCurSel();
	switch(Selection)
	{
	case 0:	//measure bar
		mode = DRAW_BAR;
		break;
	case 1:	//note tie
		mode = DRAW_TIE;
		break;
	case 2:	//Loudness (Note Velocity)
		mode = DRAW_LOUDNESS;
		{
			CParamDlg Dlg;
			Dlg.m_pCaption = "Set Loudness";
			Dlg.m_Min = 0;
			Dlg.m_Max = 127;
			Dlg.m_Val = 100;
			if(IDOK == Dlg.DoModal())
			{
				v = Dlg.m_Val;
			}
			else
				return;
		}
		break;
	case 3:	//Song Tempo
		mode = DRAW_TEMPO;
		{
			CParamDlg Dlg;
			Dlg.m_pCaption = "Set Tempo";
			Dlg.m_Min = 56;
			Dlg.m_Max = 200;
			Dlg.m_Val = 160;
			if(IDOK == Dlg.DoModal())
			{
				v = Dlg.m_Val;
			}
			else
				return;
		}
		break;
	case 4:	//change instrument
		mode = DRAW_INSTCHANGE;
		break;
	}
	m_pStaffView->SendMessage(WM_DRAWMODE,mode,v);	
}

long CMSPlayerDlg::CreateNoteMessageWord()
{
	long rv = 0;
	if(this->m_NoteDotted && !m_NoteRest)
		rv |= DRAW_NOTE_DURATION & this->m_NoteDur+2;
	else if (this->m_NoteTriplet && !m_NoteRest)
		rv |= DRAW_NOTE_DURATION & this->m_NoteDur-2;
	else
		rv |= DRAW_NOTE_DURATION & this->m_NoteDur;
	rv |= this->m_NoteAccent << DRAW_NOTE_ACCENT_SHIFT;
	rv |= this->m_NoteKey << DRAW_NOTE_KEY_SHIFT;
	rv |= this->m_NoteTrack << DRAW_NOTE_TRACK_SHIFT;
	rv |= this->m_NoteRest << DRAW_NOTE_REST_SHIFT;
	return rv;
}

void CMSPlayerDlg::OnFileNew() 
{
	// TODO: Add your command handler code here
			if(m_pSong)
			{
				m_pStaffView->m_pSong = 0;
				delete m_pSong;
			}
			extern void ResetEventcount(void);
			ResetEventcount();
			m_pSong = new CMsSong;
			m_pSong->CreateMidiInfo();
			m_pSong->m_hMidiOut = this->m_hMidiOut;
			m_pSong->m_pWndParent = this;
			CMsTimeSig *pTS = new CMsTimeSig(TIMESIG_4_4);
			m_pSong->AddObjectToSong(0, pTS);
			CMsKeySignature *pKS = new CMsKeySignature(CMAJ);
			m_pSong->AddObjectToSong(1, pKS);
			CMsTempo *pTM = new CMsTempo(160);
			m_pSong->AddObjectToSong(0, pTM);
			CMsLoudness *pLD = new CMsLoudness(127);
			m_pSong->AddObjectToSong(1, pLD);
			m_pStaffView->m_pSong = m_pSong;
			m_pStaffView->m_hMidiOut = m_hMidiOut;
			m_pStaffView->Invalidate();
}

void CMSPlayerDlg::OnFileSaveas() 
{
	// TODO: Add your command handler code here
	CFileDialog Dlg(false);
	if(IDOK == Dlg.DoModal())
	{
		CString path;

		path = Dlg.GetPathName();
		FILE *pO;
		pO = fopen(LPCTSTR(path),"wb");
		m_pSong->Save(pO);
		fclose(pO);
	}

}

void CMSPlayerDlg::OnFileSave() 
{
	// TODO: Add your command handler code here
	
}

BOOL CMSPlayerDlg::PreTranslateMessage(MSG* pMsg) 
{
	BOOL rV = TRUE;

	switch(pMsg->message)
	{
		case WM_KEYUP:
			switch(pMsg->wParam)
			{
				case VK_ESCAPE:
					m_pStaffView->SendMessage(pMsg->message, pMsg->wParam, pMsg->lParam);
					break;
				case'R':
				case 'W':
				case 'H':
				case 'Q':
				case 'E':
				case 'S':
				case 'T':
				case '3':
				case 'N':
				case 'B':
				case VK_SPACE:
				case VK_OEM_PERIOD:
					if (GetFocus()->GetDlgCtrlID() == IDC_COMBO_DECORATIONS)
						return CDialog::PreTranslateMessage(pMsg);
					else
						m_pStaffView->SendMessage(pMsg->message,pMsg->wParam,pMsg->lParam);
					break;
				default:
					return CDialog::PreTranslateMessage(pMsg);
					break;
			}
			break;
		case WM_KEYDOWN:
			switch(pMsg->wParam)
			{
				case VK_ESCAPE:
					m_pStaffView->SendMessage(pMsg->message, pMsg->wParam, pMsg->lParam);
					break;
				case VK_SPACE:
					if (GetFocus()->GetDlgCtrlID() == IDC_COMBO_DECORATIONS)
						return CDialog::PreTranslateMessage(pMsg);
					else
						m_pStaffView->SendMessage(pMsg->message,pMsg->wParam,pMsg->lParam);
					break;
				default:
					return CDialog::PreTranslateMessage(pMsg);
					break;
			}
			break;
		default:
			return CDialog::PreTranslateMessage(pMsg);
			break;
	}
	return rV;
}

void CMSPlayerDlg::OnSelchangeComboKeysig() 
{
	// TODO: Add your control notification handler code here
	m_KeySig = this->m_Combo_KeySig.GetCurSel() + 1;
	m_pStaffView->SendMessage(WM_DRAWMODE,DRAW_KEYSIG,m_KeySig);	
	
}

void CMSPlayerDlg::OnSelchangeComboTimesig() 
{
	// TODO: Add your control notification handler code here
	m_TimeSig = this->m_Combo_TimeSig.GetCurSel() + 1;
	m_pStaffView->SendMessage(WM_DRAWMODE,DRAW_TIMESIG,m_TimeSig);	
}

void CMSPlayerDlg::ChangePatch(int chan, int patch)
{
	DWMSG dwMsg;

	dwMsg.mC[2] = 0;
	dwMsg.mC[1] = (char)patch;
	dwMsg.mC[0] = MIDI_PGMCHANGE | (chan & 0x0f);
	midiOutShortMsg(m_hMidiOut,dwMsg.m_dwMsg);
}

LRESULT CMSPlayerDlg::OnPgmChange(WPARAM patch, LPARAM Instrument)
{
	int chan=0;

	chan = this->m_pSong->GetMidiInfo()->GetChannel(Instrument);
	ChangePatch(chan,patch);
	
	char *s = new char[256];
	sprintf_s(s,256,"Patch:%d  Chan:%d Inst:%d",patch,chan,Instrument);
	this->m_Static_Message.SetWindowText(s);
	delete[] s;
	return 0;
}

LRESULT CMSPlayerDlg::OnMidiInMsg(WPARAM wMsg, LPARAM timestamp)
{
	int Cmd;
	int Vel;
	int Note;
	int Chan;
	char *s = new char[256];

	Cmd = wMsg & 0xf0;
	Chan = wMsg & 0x0f;
	Note = (wMsg & 0x0ff00) >> 8;
	Vel = (wMsg & 0x0ff0000) >> 16;
	switch(Cmd)
	{
		case MIDI_NOTEOFF:
			sprintf_s(s,256,"NOTE OFF:Chan=%d Vel=%d Note=%d",Chan,Vel,Note);
			this->m_Static_Message.SetWindowText(s);
			break;
		case MIDI_NOTEON:
			sprintf_s(s,256,"NOTE ON:Chan=%d Vel=%d Note=%d",Chan,Vel,Note);
			this->m_Static_Message.SetWindowText(s);
			break;
		case MIDI_POLYPRESS:
			sprintf_s(s,256,"POLY PRESSURE:Chan=%d Pressure=%d Note=%d",Chan,Vel,Note);
			this->m_Static_Message.SetWindowText(s);
			break;
		case MIDI_CTRLCHNG:
			sprintf_s(s,256,"CONTROL CHANGE:Chan=%d Ctrl=%d Value=%d",Chan,Note,Vel);
			this->m_Static_Message.SetWindowText(s);
			break;
		case MIDI_CHNLPRESS:
			sprintf_s(s,256,"CHANNEL PRESSURE:Chan=%d Pressure=%d",Chan,Vel);
			this->m_Static_Message.SetWindowText(s);
			break;
		case MIDI_PITCHBEND:
			sprintf_s(s,256,"PITCH BEND:Chan=%d Value=%d",Chan,Vel | (Note << 7));
			this->m_Static_Message.SetWindowText(s);
			break;
		case MIDI_PGMCHANGE:
			sprintf_s(s,256,"Program Change:Chan=%d Pgm=%d",Chan,Vel);
			this->m_Static_Message.SetWindowText(s);
			break;
		case MIDI_CLOCK:
			break;
		case MIDI_START:
			break;
		case MIDI_STOP:
			break;
		case MIDI_CONTINUE:
			break;
	}
	delete[] s;
	return 0;
}

LRESULT CMSPlayerDlg::OnMidiInLongMsg(WPARAM wp, LPARAM timestamp)
{
	MIDIHDR *pMHdr = (MIDIHDR *)wp;
	int i,j,n;
	char *s = new char[256];
	char *pB = pMHdr->lpData;
	int l;

	n = pMHdr->dwBufferLength;
	for(i=0;i<n;i+= 16)
	{
		l = 0;
		for(j=0;j<16;++j)
		{
			if(j==15)
				l += sprintf_s(&s[l],256-l,"%02x",pB[i+j] & 0x0ff);
			else
				l += sprintf_s(&s[l],256-l,"%02x-",pB[i+j] & 0x0ff);
		}
		this->m_Static_Message.SetWindowText(s);
	}
	delete[] s;
	return 0;
}


void CMSPlayerDlg::OnFileDump()
{
	//This is for DEBUG.  It will dump
	//the contents of the song
	CFileDialog Dlg(false);
	if (IDOK == Dlg.DoModal())
	{
		CString path;

		path = Dlg.GetPathName();
		FILE *pO;
		pO = fopen(LPCTSTR(path), "w");
		m_pSong->Print(pO);
		fclose(pO);
	}
}

// This function is used to update the combo box selection variables.
void CMSPlayerDlg::UpdateComboBoxes()
{
	m_NoteKey = AccedentLUT[m_Combo_Accident.GetCurSel()];
	int sel;
	m_NoteAccent = m_Combo_Decorations.GetCheck(COMBO_DECOR_ACCENT);
	sel = m_Combo_Decorations.GetCurSel();
	switch (sel)
	{
		case COMBO_DECOR_NONE:
			m_NoteDotted = 0;
			m_NoteTriplet = 0;
			break;
		case COMBO_DECOR_DOT:
			m_NoteDotted = 1;
			m_NoteTriplet = 0;
			break;
		case COMBO_DECOR_TRIPLET:
			m_NoteDotted = 0;
			m_NoteTriplet = 1;
			break;
	}
	m_NoteTrack = m_Combo_Instrument.GetCurSel() + 1;		//Instrument number needs +1
	this->SendMessage(WM_PGMCHANGE, m_pSong->GetMidiInfo()->GetPatch(m_NoteTrack), m_NoteTrack);
	m_NoteDur = NoteDurLut[m_Combo_NoteType.GetCurSel()];
}
