#include "stdafx.h"
#include "resource.h"
#include <stdio.h>
#include <string.h>
#include "MidiInfo.h"
#include "MSPlayerDlg.h"
#include "MusicStudioFileFormat.h"

CMidiInfo::CMidiInfo()
{
    //ctor
	memset(this->m_SongTitle,0,sizeof(m_SongTitle));
	strcpy(m_SongTitle,"Song Title");
}

CMidiInfo::~CMidiInfo()
{
    //dtor
}

extern char * GenMidPatchNames[];


int Patch[15] = {
	1,		//0	Percusion
	23,		//1	Harmonica
	67,		//2 Sax
	27,		//3	Gutar
	72,		//4 Clarinet
	74,		//5 Flute
	90,		//6 Pads
	1,		//7	Piano
	17,		//8	Organ
	57,		//9	Trumpet
	51,		//10	Strings
	12,		//11	Vibes
	15,		//12	Bells
	47,		//13	Harp
	39		//14	Bass
};

int Channel[15] = {
	9,		//0	channel 10
	1,		//1
	2,		//2
	3,		//3
	4,		//4
	5,		//5
	6,		//6
	7,		//7
	8,		//8
	10,		//9
	11,		//10
	12,		//11
	13,		//12
	14,		//13
	15		//14
};

int Range[15] = {
	2,		//0
	3,		//1
	3,		//2
	3,		//3
	3,		//4
	3,		//5
	3,		//6
	3,		//7
	3,		//8
	3,		//9
	3,		//10
	3,		//11
	3,		//12
	3,		//13
	1		//14
};

void CMidiInfo::Create()
{
	CMSPlayerDlg *pMain;
	pMain = (CMSPlayerDlg *)AfxGetApp()->m_pMainWnd;
	CBitmap *pBM = &pMain->m_BmInst1;
	for(int Inst=1;Inst<16;++Inst)	//from instrument 1 to 15
	{
		char *pName;
		if(9 == Channel[Inst-1])
			pName = "Percussion";
		else
			pName = GenMidPatchNames[Patch[Inst-1] ];
		this->SetInstrument(Inst,pName,Patch[Inst-1], Channel[Inst-1],Range[Inst-1]);
		this->m_Colors[Inst - 1].SetBitmap(pBM++);
	}
}

void CMidiInfo::Create(char *MidiInfo)
{
	CMSPlayerDlg *pMain;
	pMain = (CMSPlayerDlg *)AfxGetApp()->m_pMainWnd;
	CBitmap *pBM = &pMain->m_BmInst1;
	MIDI_INFO *pMI = (MIDI_INFO *)MidiInfo;
	int Inst;
	for(Inst=1;Inst<16;++Inst)
	{
		int i = Inst-1;
		this->m_Colors[Inst - 1].SetBitmap(pBM++);
		m_tracks[i].m_Channel = pMI->chan_range[i] & 0x0f;
		if(m_tracks[i].m_Channel == 0)
			m_tracks[i].m_Channel = 15;
		else
			m_tracks[i].m_Channel -= 1;
		m_tracks[i].m_Range = ( pMI->chan_range[i] & 0xf0) >> 4;
		m_tracks[i].m_Patch = pMI->patch[i];
		if(m_tracks[i].m_Channel == 9)	//midi channel 10
			strcpy(m_tracks[i].m_InstrumentName,"Percusion");
		else
			strcpy(m_tracks[i].m_InstrumentName,pMI->Name[i].Name);
		int j;
		unsigned short mask = (1 << i);
		for(j=0;j<4;++j)
			m_tracks[i].m_Enable[j] = (pMI->TrackEnables[j] & mask)?1:0;
	}
	strcpy(this->m_SongTitle,pMI->Title);
}

void CMidiInfo::SetInstrument(int instIndex,char *n,unsigned short P,unsigned short C, unsigned short R)
{
    if(instIndex < 16 && instIndex > 0)
    {
		int index = instIndex - 1;
        m_tracks[index].m_Channel = C;
        m_tracks[index].m_Patch = P;
        m_tracks[index].m_Range = R;
        strcpy(m_tracks[index].m_InstrumentName,n);
    }
}

bool CMidiInfo::IsTrackEnable(int block, int track)
{
	//---------------------------------------
	// IsTrackEnable
	//
	//	Determines if queried track is enabled.
	//
	// parameters:
	//	block.....block of enables to check
	//	track.....Instrument number, 1-15
	//
	// returns true if enabled
	//-------------------------------------------
    bool rV = false;
    if(block < 4 && track < 16 && track > 0)
    {
		//subtract 1 from track (inst) to get index
        rV = m_tracks[track-1].m_Enable[block]?true:false;
    }
    return rV;
}

void CMidiInfo::SetTitle(char *n)
{
    strcpy(m_SongTitle,n);
}

void CMidiInfo::Print(FILE *pO)
{

}

void CMidiInfo::SetTrackEnable(int bank,unsigned short v,short Inst)
{
	//-----------------------------------------
	// SetTrackEnable
	//	This function is used to set
	// or clear the track enable bit
	// for each instrument
	//
	// parameters:
	//		bank....track enable bank 0..3
	//		v.......value to set bit 0..1
	//		Inst....Instrument number 1..15
	//----------------------------------------
	//subtract 1 from Inst to get index
	m_tracks[Inst-1].m_Enable[bank] = (char)v;
}

void CMidiInfo::Save(FILE *pO)
{
	int i,j;

	for(j=0;j<15;++j)
	{
		for(i=0;i<9;++i)
		{
			fputc(this->m_tracks[j].m_InstrumentName[i],pO);
		}
		fputc(0,pO);	//Null terminator for string
	}
	for (j = 0; j < 15; ++j)
	{
		int chan = m_tracks[j].m_Channel;
		++chan;
		if (chan == 16) chan = 0;
		fputc(chan | (m_tracks[j].m_Range << 4), pO);
	}
	for(j=0;j<15;++j)
		fputc(m_tracks[j].m_Patch,pO);
	for(i=0;i<20;++i) fputc(0,pO);	///pointers to lyrics
	for(i=0;i<32;++i) fputc(m_SongTitle[i],pO);
	for(i=0;i<8;++i) fputc(0xff,pO);	//track enables
}

void CMidiInfo::SetInstName(int inst, char *pName)
{
	strcpy(this->m_tracks[inst-1].m_InstrumentName,pName);
}
