// MsEvent.cpp: implementation of the CMsEvent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MsEvent.h"
#include "MsObject.h"
#include "MsNote.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
static int EventCount = 0;

void ResetEventcount(void)
{
	EventCount = 0;
}

void SetEventCount(int v)
{
	EventCount = v;
}

CMsEvent::CMsEvent()
{
	m_pNext = 0;
	m_pPrev = 0;
	m_EventObjectListHead = 0;
	m_EventObjectListTail = 0;
	m_Selected = 0;
	m_Index = EventCount++;
}

CMsEvent::~CMsEvent()
{
	if(m_EventObjectListHead)
	{
		CMsObject *pObj,*pObjDel;

		pObj = m_EventObjectListHead;
		while(pObj)
		{
			pObjDel = pObj;
			pObj = pObj->GetNext();
			delete pObjDel;
		}
	}
}

void CMsEvent::Print(FILE *pO)
{
	CMsObject *pObj = this->GetHead();
	fprintf(pO, "=========Event %d Begin ========\n", m_Index);
	while(pObj)
	{
		pObj->Print(pO);
		pObj = pObj->GetNext();
	}
	fprintf(pO,"------------ event end-------------\n");
}

void CMsEvent::AddObjectAtEnd(CMsObject *pO)
{
	if(m_EventObjectListHead == 0)
	{
		m_EventObjectListHead = pO;
		m_EventObjectListTail = pO;
	}
	else
	{
		pO->SetPrev(m_EventObjectListTail);
		m_EventObjectListTail->SetNext(pO);
		m_EventObjectListTail = pO;
	}
}

void CMsEvent::AddObjectAtStart(CMsObject *pO)
{
	if(m_EventObjectListTail == 0)
	{
		m_EventObjectListHead = pO;
		m_EventObjectListTail = pO;
	}
	else
	{
		pO->SetNext(m_EventObjectListHead);
		m_EventObjectListHead->SetPrev(pO);
		m_EventObjectListHead = pO;
	}
}

void CMsEvent::Draw(CDC *pDC, int event, int maxevent)
{
	CMsObject *pObj = this->GetHead();
	int x = EVENT_OFFSET + EVENT_WIDTH * event;
	if(this->IsSelected())
	{
		CDC dc;		//memory DC
		CBitmap bm;

		CRect r,srect;
		pDC->GetWindow()->GetClientRect(&r);
		srect.SetRect(x,r.top,x+EVENT_WIDTH,r.bottom);
		bm.CreateBitmap(EVENT_WIDTH,r.Height(),1,32,0);
		dc.CreateCompatibleDC(pDC);
		dc.SelectObject(&bm);
		CBrush br;
		br.CreateSolidBrush(RGB(50,180,60));
		dc.FillRect(CRect(0,0,EVENT_WIDTH,r.Height()),&br);
		dc.BitBlt(0,0,EVENT_WIDTH,r.Height(),pDC,x,r.top,SRCAND);
		pDC->BitBlt(x,r.top,EVENT_WIDTH,r.Height(),&dc,0,0,SRCCOPY);
	}
	while(pObj)
	{
		pObj->Draw(pDC,event,maxevent);
		pObj = pObj->GetNext();
	}
}

CMsObject *CMsEvent::ObjectAlreadyHere(CMsObject *pObj)
{
	int loop = 1;
	ObjectTypes obj,sobj;
	CMsObject *rpObj=0;

	CMsObject *pH = this->GetHead();
	while(pH && loop)
	{
		if(pH->GetType() == pObj->GetType())
		{
			switch(pH->GetType())
			{
				case MSOBJ_KEYSIG:
				case MSOBJ_BAR:
				case MSOBJ_LOUDNESS:
				case MSOBJ_TEMPO:
				case MSOBJ_TIMESIG:
					rpObj = pH;
					loop = 0;
					break;
				case MSOBJ_END:
					break;
				case MSOBJ_NOTE:
					obj.pObj = pObj;
					sobj.pObj = pH;
					if(obj.pNote->GetPitch() == sobj.pNote->GetPitch())	//found it
					{
						if(obj.pNote->GetTrack() == sobj.pNote->GetTrack())
						{
							rpObj=pH;
							loop = 0;
						}
					}
					break;
				case MSOBJ_REPEATEND:
					break;
				case MSOBJ_REPEATSTART:
					break;
			}
		}
		pH = pH->GetNext();
	}
	return rpObj;
}

int CMsEvent::RemoveObject(CMsObject *pObj)
{
	if(pObj == m_EventObjectListHead)
	{
		m_EventObjectListHead = pObj->GetNext();
		if(m_EventObjectListHead)
			m_EventObjectListHead->SetPrev(0);
		else
			m_EventObjectListTail = 0;
	}
	else if(pObj == m_EventObjectListTail)
	{
		m_EventObjectListTail = pObj->GetPrev();
		if(m_EventObjectListTail)
			m_EventObjectListTail->SetNext(0);
		else
			m_EventObjectListHead = 0;
	}
	else
	{
		pObj->GetNext()->SetPrev(pObj->GetPrev());
		pObj->GetPrev()->SetNext(pObj->GetNext());
	}
	delete pObj;
	return 0;
}


// Function returns Note of a given pitch
CMsNote * CMsEvent::FindNote(int Note, int Accidental)
{
	CMsObject *pObj = this->m_EventObjectListHead;
	CMsNote *pN=0;
	int loop = 1;
	while (pObj && loop)
	{
		if (pObj->GetType() == MSOBJ_NOTE)
		{
			pN = (CMsNote *)pObj;
			if (pN->GetPitch() == Note && pN->GetKey() == Accidental)
				loop = 0;
			else
			{
				pObj = pObj->GetNext();
				pN = 0;
			}
		}
		else
		{
			pObj = pObj->GetNext();
		}
	}
	return pN;
}


// Determines if the paricular object is in this event
CMsObject *CMsEvent::ContainsObjectType(int ObjectType)
{
	CMsObject *pObj = this->m_EventObjectListHead;
	int loop = 1;

	while (loop && pObj)
	{
		if (pObj->GetType() == ObjectType)
		{
			loop = 0;
		}
		else
			pObj = pObj->GetNext();
	}
	return pObj;
}

int CMsEvent::AreObjectsSelected()
{
	int Count = 0;
	CMsObject *pObj = this->GetHead();
	while (pObj)
	{
		if (pObj->IsSelected())
			++Count;
		pObj = pObj->GetNext();
	}
	return Count;
}

CMsObject * CMsEvent::GetFirstSelectedObject()
{
	CMsObject *pObj = GetHead();
	int loop=1;
	while (pObj && loop)
	{
		if (pObj->IsSelected())
			loop = 0;
		else
			pObj = pObj->GetNext();
	}
	return pObj;
}

CMsObject * CMsEvent::GetNextSelectedObject(CMsObject * pLastSelectedObject)
{
	CMsObject *pObj = pLastSelectedObject->GetNext();
	int loop = 1;
	while (pObj && loop)
	{
		if (pObj->IsSelected())
			loop = 0;
		else
			pObj = pObj->GetNext();
	}
	return pObj;
}


CMsObject * CMsEvent::ContainsRepeatObject()
{
	CMsObject *pObj = GetHead();
	int loop=1;
	int type;

	while (loop && pObj)
	{
		type = pObj->GetType();
		if (MSOBJ_REPEATEND == type || MSOBJ_REPEATSTART == type)
			loop = 0;
		else
		{
			pObj = pObj->GetNext();
		}
	}
	return pObj;
}
