// MsKeySignature.cpp: implementation of the CMsKeySignature class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MsKeySignature.h"
#include "MsSharp.h"
#include "MsNote.h"
#include "MsFlat.h"
#include "MusicStudioFileFormat.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMsKeySignature::CMsKeySignature():CMsObject(MSOBJ_KEYSIG)
{
	this->m_KeySignature = 0;
	this->SetKeySigCorrection();
}

CMsKeySignature::CMsKeySignature(int k):CMsObject(MSOBJ_KEYSIG)
{
	this->m_KeySignature = k;
	this->SetKeySigCorrection();
}

CMsKeySignature::~CMsKeySignature()
{

}

char *KeySigStringTab[17] = {
	"ERROR",
	"C MAJ",
	"G MAJ",
	"D MAJ",
	"A MAJ",
	"E MAJ",
	"B MAJ",
	"F# MAJ",
	"C# MAJ",
	"F MAJ",
	"Bb MAJ",
	"Eb MAJ",
	"Ab MAJ",
	"Db MAJ",
	"Gb MAJ",
	"Cb MAJ"
};

void CMsKeySignature::Print(FILE *pO)
{
	fprintf(pO,"Key Signature:%s\n",KeySigStringTab[this->m_KeySignature]);
}

void CMsKeySignature::Draw(CDC *pDC, int event, int maxevent)
{
	CMsSharp sharp;
	CMsFlat flat;
	extern int NoteToPosition(int Note);
	COLORREF color;

	if (IsSelected())
		color = RGB(255, 0, 0);
	else
		color = RGB(0, 0, 0);
	switch(m_KeySignature)
	{
		case CMAJ:	///do nothing
			break;
		case CSMAJ:	///seven shrps
			sharp.Draw(pDC,color,EVENT_OFFSET+EVENT_WIDTH*event+24,NoteToPosition(NOTE_B2));	//base staff
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+24,NoteToPosition(NOTE_B2+24));	//trebble staff
		case FSMAJ:	///six sharps
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+12,NoteToPosition(NOTE_E2+12));	//base staff
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+20,NoteToPosition(NOTE_E2+24));	//trebble staff
		case BMAJ:	///five sharps
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+16,NoteToPosition(NOTE_A2));	//base staff
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+16,NoteToPosition(NOTE_A2+24));	//trebble staff
		case EMAJ:	///four D sharps
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+20,NoteToPosition(NOTE_D2+12));	//base staff
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+12,NoteToPosition(NOTE_D2+36));	//trebble staff
		case AMAJ:	//three G sharps
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event,NoteToPosition(NOTE_G2+12));	//base staff
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+8,NoteToPosition(NOTE_G2+36));		//trebble staff
		case DMAJ:	//two C sharps
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+4,NoteToPosition(NOTE_C2+12));	//base staff
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+4,NoteToPosition(NOTE_C2+36));	//trebble staff
		case GMAJ:	///one F sharp
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+8,NoteToPosition(NOTE_F2+12));	//base staff
			sharp.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event,NoteToPosition(NOTE_F2+36));	//trebble staff
			break;
		case CFMAJ:	//seven flats
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event,NoteToPosition(NOTE_F2+12));	//bass staff
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event,NoteToPosition(NOTE_F2+36));	//trebble staff
		case GFMAJ:	//six flats
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+24,NoteToPosition(NOTE_C2+12));	//bass staff
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+24,NoteToPosition(NOTE_C2+36));	//trebble staff
		case DFMAJ:	//five flats
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+24,NoteToPosition(NOTE_G2));	//bass staff
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+24,NoteToPosition(NOTE_G2+24));	//trebble staff
		case AFMAJ:	//four flats
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event +16,NoteToPosition(NOTE_D2+12));	//bass staff
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+16,NoteToPosition(NOTE_D2+36));	//trebble staff
		case EFMAJ:	//three flats
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+16,NoteToPosition(NOTE_A2));	//bass staff
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+16,NoteToPosition(NOTE_A2+24));	//trebble staff
		case BFMAJ:	//two flats
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+8,NoteToPosition(NOTE_E2+12));	//bass staff
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+8,NoteToPosition(NOTE_E2+36));	//trebble staff
		case FMAJ:	//one flat
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+8,NoteToPosition(NOTE_B2));	//bass staff
			flat.Draw(pDC, color,EVENT_OFFSET+EVENT_WIDTH*event+8,NoteToPosition(NOTE_B2+24));	//trebble staff
			break;
	}
}


void CMsKeySignature::SetKeySigCorrection(void)
{
	int KeySig = this->m_KeySignature;
	int i;
	for(i=0;i<12;++i)
	{
		m_KeySigCorrection[i] = 0;
		switch(KeySig)
		{
			case CMAJ:
				break;
			case CSMAJ:
				if(i == NOTE_C)
					m_KeySigCorrection[i] = 1;
			case FSMAJ:
				if(i == NOTE_F)
					m_KeySigCorrection[i] = 1;
			case BMAJ:
				if(i == NOTE_A)
					m_KeySigCorrection[i] = 1;
			case EMAJ:
				if(i == NOTE_D)
					m_KeySigCorrection[i] = 1;
			case AMAJ:
				if(i == NOTE_G)
					m_KeySigCorrection[i] = 1;
			case DMAJ:	//two sharps
				if(i == NOTE_F)
					m_KeySigCorrection[i] = 1;
			case GMAJ:	//one sharp
				if(i == NOTE_C)
					m_KeySigCorrection[i] = 1;
				break;
			case CFMAJ:
				if(i == NOTE_F)
					m_KeySigCorrection[i] = -1;
			case GFMAJ:
				if(i == NOTE_C)
					m_KeySigCorrection[i] = -1;
			case DFMAJ:
				if(i == NOTE_G)
					m_KeySigCorrection[i] = -1;
			case AFMAJ:
				if(i == NOTE_D)
					m_KeySigCorrection[i] = -1;
			case EFMAJ:
				if(i == NOTE_A)
					m_KeySigCorrection[i] = -1;
			case BFMAJ:
				if(i == NOTE_E)
					m_KeySigCorrection[i] = -1;
			case FMAJ:
				if(i == NOTE_B)
					m_KeySigCorrection[i] = -1;
				break;
		}
	}
}

int CMsKeySignature::GetKeySigCorrection(int note, int accidental)
{
	int n = note % 12;
	switch(accidental)
	{
		case ACCIDENT_INKEY:
			note += m_KeySigCorrection[n];
			break;
		case ACCIDENT_NATURAL:
			break;
		case ACCIDENT_SHARP:
			note += 1;
			break;
		case ACCIDNET_FLAT:
			note -= 1;
			break;
	}
	return note;
}

CMsObject * CMsKeySignature::Copy()
{
	CMsObject *pOb = 0;
	CMsKeySignature *pB = new CMsKeySignature;
	*pB = *this;
	pOb = pB;
	return pOb;
}

void CMsKeySignature::Save(FILE *pO)
{
	fputc(TOKEN_KEY_SIGNATURE,pO);
	fputc(0x01,pO);
	fputc(this->m_KeySignature,pO);
}
