// MsNote.cpp: implementation of the CMsNote class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MsNote.h"
#include "MusicStudioPlayer.h"
#include "MSPlayerDlg.h"
#include "MusicStudioFileFormat.h"
#include "MsSharp.h"
#include "Msflat.h"
#include "MsNatural.h"
#include "WindowMessages.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMsNote::CMsNote():CMsObject(MSOBJ_NOTE)
{
    m_Pitch = 0;
    m_Rest = 0;
    m_TieBeg = 0;
    m_TieEnd = 0;
	m_Stacato = 0;
    m_Accent = 0;
	m_pEvParent = 0;
	m_UpsideDown = 0;
}

CMsNote::CMsNote(CMsEvent *pP):CMsObject(MSOBJ_NOTE)
{
    m_Pitch = 0;
    m_Rest = 0;
    m_TieBeg = 0;
    m_TieEnd = 0;
	m_Stacato = 0;
    m_Accent = 0;
	m_pEvParent = pP;
	m_UpsideDown = 0;
}

CMsNote::~CMsNote()
{

}

DUR DurTab[0x15] = {
	-1,0,0,1,0,0,0,"NA",				//0
	5,0,1,1,3,1,4,"32nd Trip",		//1
	-1,0,0,1,0,0,0,"NA",				//2
	5,0,0,1,3,1,6,"ThirtySecond",		//3 thirty second
	4,0,1,1,2,1,8,"16th Trip",		//4
	5,1,0,1,3,1,9,"Dotted TS",		//5 Dotted Thirty Second
	4,0,0,1,2,1,12,"Sixteenth",		//6 sixteenth
	3,0,1,1,1,1,16,"8th Trip",		//7
	4,1,0,1,2,1,18,"Dotted Sixteenth",//8	Dotted Sixteenth
	3,0,0,1,1,1,24,"Eighth",			//9 eighth
	2,0,1,1,0,1,32,"1/4 Trip",		//10
	3,1,0,1,1,1,36,"Dotted Eighth",	//11 Dotted Eighth
	2,0,0,1,0,1,48,"Quarter",			//12 quarter
	1,0,1,0,0,1,64,"1/2 Trip",		//13
	2,1,0,1,0,1,72,"Dotted Quarter",	//14 Dotted Quarter Note
	1,0,0,0,0,1,96,"Half",			//15 half
	0,0,1,0,0,0,128,"Whole Trip",		//16
	1,1,0,0,0,1,144,"Dptted Half",	//17 Dotted Halfe
	0,0,0,0,0,0,192,"Whole",			//18 whole
	-1,0,0,0,0,0,0,"NA",				//19
	0,1,0,0,0,0,288,"Dotted Whole"	//20 dotted whole
};

char *NoteLUT[12] = {
	"C",
	"C#",
	"D",
	"D#",
	"E",
	"F",
	"F#",
	"G",
	"G#",
	"A",
	"A#",
	"B"
};

int KeyLUT[4] = {
	' ',
	'N',
	'#',
	'b'
};


// This function converts the note value to a string
int CMsNote::NoteToString(char * pInString, int size)
{
	int note = this->GetPitch() % 12;
	int oct = this->GetPitch() / 12;

	int l = sprintf_s(pInString, size, "%s%d%c", NoteLUT[note], oct, KeyLUT[GetKey()]);
	return l;
}

void CMsNote::Print(FILE *pO)
{
	int Note,Oct;
	Note = this->m_Pitch % 12;
	Oct = this->m_Pitch / 12;
	fprintf(pO,"NOTE:%4s%d%c %16s Track:%2d Tie:%c:%c %s %s\n",NoteLUT[Note],Oct,KeyLUT[m_Key],DurTab[m_Duration].pName,m_Track,m_TieBeg?'B':' ',m_TieEnd?'E':' ',m_Rest?"REST":"",m_Accent?"ACCENT":"");

}

int CMsNote::GetDuration(void)
{
	return m_Duration;
}

int NotePos[12] = {
	24,	//C
	24,	//C#
	20,	//D
	20,	//D#
	16,	//E
	12,	//F
	12,	//F#
	8,	//G
	8,	//G#
	4,	//A
	4,	//A#
	0	//B
};

int NoteToPosition(int Note)
{
	return NotePos[Note % 12] + 28 * (7 - (Note / 12)) + (STAVE_OFFSET - 44);
}

void CMsNote::Draw(CDC *pDC, int event, int maxevent)
{
	CPoint End,Start;
	CPen pen,*oldpen;
	CBrush brush,*oldbrush;
	COLORREF color;
	int i,n,f;

	color = this->m_pMidiInfo->GetInstColor(m_Track)->GetColor();
	if (IsSelected())
		color ^= 0x00ffffff;
	pen.CreatePen(PS_SOLID,1,color);
	if(IsSolid() || IsRest())
		brush.CreateSolidBrush(color);
	else
		brush.CreateSolidBrush(RGB(255,255,255));
	oldpen = pDC->SelectObject(&pen);
	oldbrush = pDC->SelectObject(&brush);
	int notev = this->m_Pitch;
	//--------------------------
	// the value of notev will
	// range from 24 to 60
	//------------------------
	notev = NotePos[notev%12] + 28 * (7 - (notev / 12)) + (STAVE_OFFSET - 44);
	CRect r;
	if(IsRest())	///draw rest on staff
	{
		switch(m_Duration)
		{
			case REST_THIRTYSECOND:
				{
					CBitmap *pOldBm,Rest;
					int x,y;

					Rest.LoadBitmap(IDB_INVTHIRTY);
					CDC dc;
					dc.CreateCompatibleDC(pDC);
					pOldBm = dc.SelectObject(&((CMSPlayerDlg *)theApp.m_pMainWnd)->m_BmThirtySecRest);
					x = EVENT_OFFSET+EVENT_WIDTH*event;
					y = notev;
					pDC->BitBlt(x,y,10,20,&dc,0,0,SRCAND);
					dc.SelectObject(&Rest);
					ChangeRestColor(&dc,color,10,20);
					pDC->BitBlt(x,y,10,20,&dc,0,0,SRCPAINT);
					pDC->SelectObject(pOldBm);
				}
				break;
			case REST_SIXTEENTH:
				{
					CBitmap *pOldBm,Rest;
					int x,y;

					Rest.LoadBitmap(IDB_INVSIXTEENTH);
					CDC dc;
					dc.CreateCompatibleDC(pDC);
					pOldBm = dc.SelectObject(&((CMSPlayerDlg *)theApp.m_pMainWnd)->m_BmSixteenthRest);
					x = EVENT_OFFSET+EVENT_WIDTH*event;
					y = notev;
					pDC->BitBlt(x,y,8,20,&dc,0,0,SRCAND);
					dc.SelectObject(&Rest);
					ChangeRestColor(&dc,color,8,20);
					pDC->BitBlt(x,y,8,20,&dc,0,0,SRCPAINT);
					pDC->SelectObject(pOldBm);
				}
				break;
			case REST_EIGTH:
				{
					CBitmap *pOldBm,Rest;
					int x,y;

					Rest.LoadBitmap(IDB_INVEIGTH);
					CDC dc;
					dc.CreateCompatibleDC(pDC);
					pOldBm = dc.SelectObject(&((CMSPlayerDlg *)theApp.m_pMainWnd)->m_BmEigthRest);
					x = EVENT_OFFSET+EVENT_WIDTH*event;
					y = notev;
					pDC->BitBlt(x,y,8,20,&dc,0,0,SRCAND);
					dc.SelectObject(&Rest);
					ChangeRestColor(&dc,color,8,20);
					pDC->BitBlt(x,y,8,20,&dc,0,0,SRCPAINT);
					pDC->SelectObject(pOldBm);
				}
				break;
			case REST_QUARTER:
				{
					CBitmap *pOldBm,Rest;
					int x,y;

					Rest.LoadBitmap(IDB_QUARTERREST_INVERT);
					CDC dc;
					dc.CreateCompatibleDC(pDC);
					pOldBm = dc.SelectObject(&((CMSPlayerDlg *)theApp.m_pMainWnd)->m_BmQuarterRest);
					x = EVENT_OFFSET+EVENT_WIDTH*event;
					y = notev;
					pDC->BitBlt(x,y,8,20,&dc,0,0,SRCAND);
					dc.SelectObject(&Rest);
					ChangeRestColor(&dc,color,8,20);
					pDC->BitBlt(x,y,8,20,&dc,0,0,SRCPAINT);
					pDC->SelectObject(pOldBm);
				}
				break;
			case REST_HALF:
				if(NeedsLine())
				{
					r.SetRect(EVENT_OFFSET+EVENT_WIDTH*event,notev,EVENT_OFFSET+EVENT_WIDTH*event+8,notev+4);
					pDC->MoveTo(EVENT_OFFSET+EVENT_WIDTH*event-2,notev+4);
					pDC->LineTo(EVENT_OFFSET+EVENT_WIDTH*event+10,notev+4);
				}
				else
				{
					r.SetRect(EVENT_OFFSET+EVENT_WIDTH*event,notev,EVENT_OFFSET+EVENT_WIDTH*event+8,notev+HalfRestOffset());
				}
				pDC->Rectangle(&r);
				break;
			case REST_WHOLE:
				r.SetRect(EVENT_OFFSET+EVENT_WIDTH*event,notev,EVENT_OFFSET+EVENT_WIDTH*event+8,notev+WholeRestOffset());
				pDC->Rectangle(&r);
				break;
		}
	}
	else	///draw note on staff
	{
		r.SetRect(EVENT_OFFSET+EVENT_WIDTH*event,notev,EVENT_OFFSET+EVENT_WIDTH*event+10,notev+8);
		pDC->Ellipse(&r);
		if((n = NeedsLine()) )
		{
			int y;
			if(n < 0)
			{
				f = -1;
				n = -n;
			}
			else
				f = 1;
			for(i=0;i<n;++i)
			{
				if(IsOnLine(m_Pitch))
					y = notev+i*8*f;
				else
					y = notev+4+i*8*f;
				pDC->MoveTo(28 +EVENT_WIDTH*event,y);
				pDC->LineTo(44+EVENT_WIDTH*event,y);
			}
		}
		if(NeedsTail())
		{
			if(m_UpsideDown)
			{
				pDC->MoveTo(EVENT_OFFSET+EVENT_WIDTH*event,notev+4);
				pDC->LineTo(EVENT_OFFSET+EVENT_WIDTH*event,notev+28);
			}
			else
			{
				pDC->MoveTo(EVENT_OFFSET+EVENT_WIDTH*event+9,notev+4);
				pDC->LineTo(EVENT_OFFSET+EVENT_WIDTH*event+9,notev-20);
			}
			if((n=NeedsFlags()))
			{
				for(i=0;i<n;++i)
				{
					if(m_UpsideDown)
					{
						pDC->MoveTo(EVENT_OFFSET+EVENT_WIDTH*event,notev+28-(i*5));
						pDC->LineTo(EVENT_OFFSET+EVENT_WIDTH*event+4,notev+24-(i*5));
					}
					else
					{
						pDC->MoveTo(EVENT_OFFSET+EVENT_WIDTH*event+9,notev-20+i*5);
						pDC->LineTo(EVENT_OFFSET+EVENT_WIDTH*event+13,notev-14+i*5);
					}
				}
			}
		}
		if(DurTab[m_Duration].Dotted)
		{
			pDC->SetPixel(EVENT_OFFSET+EVENT_WIDTH*event+12,notev+6,color);
			pDC->SetPixel(EVENT_OFFSET+EVENT_WIDTH*event+13,notev+6,color);
			pDC->SetPixel(EVENT_OFFSET+EVENT_WIDTH*event+11,notev+6,color);
			pDC->SetPixel(EVENT_OFFSET+EVENT_WIDTH*event+12,notev+7,color);
			pDC->SetPixel(EVENT_OFFSET+EVENT_WIDTH*event+12,notev+5,color);
		}
		else if (DurTab[m_Duration].Triplet)
		{
			int x;
			x = EVENT_OFFSET+EVENT_WIDTH*event;

			pDC->SetPixel(x+12,notev+6,color);
			pDC->SetPixel(x+13,notev+6,color);
			pDC->SetPixel(x+14,notev+7,color);
			pDC->SetPixel(x+13,notev+8,color);
			pDC->SetPixel(x+14,notev+9,color);
			pDC->SetPixel(x+13,notev+10,color);
			pDC->SetPixel(x+12,notev+10,color);
		}
		if(IsAccent())
		{
			if(IsUpsideDown())
			{
				pDC->MoveTo(EVENT_OFFSET+EVENT_WIDTH*event,notev-4);
				pDC->LineTo(EVENT_OFFSET+EVENT_WIDTH*event+8,notev-7);
				pDC->LineTo(EVENT_OFFSET+EVENT_WIDTH*event,notev-10);
			}
			else
			{
				pDC->MoveTo(EVENT_OFFSET+EVENT_WIDTH*event,notev+12);
				pDC->LineTo(EVENT_OFFSET+EVENT_WIDTH*event+8,notev+15);
				pDC->LineTo(EVENT_OFFSET+EVENT_WIDTH*event,notev+18);
			}
		}	//end of if(IsAccent())
		if(IsTieend())
		{
			CMsNote *pN;
			pN = this->FindTieBegin();
			if(pN)
			{
				CMsEvent *pEv = pN->m_pEvParent;
				CRect r;
				int widthEv = m_pEvParent->m_Index - pEv->m_Index;
				int width = EVENT_WIDTH * widthEv;
				int x = 5 + EVENT_OFFSET + (event-widthEv) * EVENT_WIDTH;
				r.SetRect(x,notev,x+width,notev+16);
				if(this->m_UpsideDown)
				{
					End = CPoint(x+1,notev-8);
					Start = CPoint (x+width-1,notev-8);
				}
				else
				{
					Start= CPoint (x+1,notev+8);
					End = CPoint (x+width-1,notev+8);
				}
				pDC->Arc(r,Start,End);
			}
		}
		switch(this->m_Key)
		{
			case ACCIDENT_INKEY:
				break;
			case ACCIDENT_NATURAL:
				{
					CMsNatural nat;
					nat.Draw(pDC,color,EVENT_OFFSET+EVENT_WIDTH*event-8,notev);
				}
				break;
			case ACCIDENT_SHARP:
				{
					CMsSharp sharp;
					sharp.Draw(pDC,color,EVENT_OFFSET+EVENT_WIDTH*event-8,notev);
				}
				break;
			case ACCIDNET_FLAT:
				{
					CMsFlat flat;
					flat.Draw(pDC,color,EVENT_OFFSET+EVENT_WIDTH*event-8,notev);
				}
				break;
		}
	}
	pDC->SelectObject(oldpen);
	pDC->SelectObject(oldbrush);
}

int CMsNote::NeedsLine()
{
	//-------------------------
	// Middle C is 0x3C
	//-------------------------
	int rV = 0;

	if(m_Pitch == 0x3c) rV = 1;
	else if(m_Pitch < 0x29)
	{
		rV = (m_Pitch - 0x29)/2;

	}
	else if(m_Pitch > 0x4f)
	{
		int notev = NotePos[m_Pitch%12] + 28 * (7 - (m_Pitch / 12)) + (STAVE_OFFSET - 44);
		rV = (STAVE_OFFSET - notev)/8;
	}
	return rV;
}

bool CMsNote::NeedsTail()
{
	bool rV = DurTab[m_Duration].Tail?true:false;
	return rV;
}

int CMsNote::NeedsFlags()
{
	int flags = DurTab[m_Duration].Flags;
	return flags;
}

bool CMsNote::IsSolid()
{
	return DurTab[m_Duration].Solid?true:false;
}

static short OnLine[12] = {
	1,	///C
	1,	///C#
	0,	///D
	0,	///D#
	1,	///E
	0,	///F
	0,	///F#
	1,	///G
	1,	///G#
	0,	///A
	0,	///A#
	1	///B
};

int IsOnLine(int pitch)
{
	int rV;
	int oct = pitch/12;
	int noteonline = OnLine[pitch%12];
	if(oct & 1) noteonline ^= 1;
	rV = noteonline?1:0;
	return rV;

}

bool CMsNote::IsTriplet()
{
	bool rV = false;
	if(DurTab[this->m_Duration].Triplet) rV = true;
	return rV;
}

CMsNote * CMsNote::FindTieBegin()
{
	CMsEvent *pEv = this->m_pEvParent;
	ObjectTypes pObj;
	CMsNote *pMsN = 0;

	int loop = 1;

	pEv = (CMsEvent *)pEv->GetPrev();
	while(pEv && loop)
	{
		pObj.pObj = pEv->GetHead();
		while(pObj.pObj && loop)
		{
			switch(pObj.pObj->GetType())
			{
				case MSOBJ_NOTE:
					if((m_Pitch == pObj.pNote->GetPitch()) && pObj.pNote->IsTieBeg())
					{
						pMsN = pObj.pNote;
						loop = 0;
					}
					else
						pObj.pObj = pObj.pObj->GetNext();
					break;
				default:
					pObj.pObj = pObj.pObj->GetNext();
					break;
			}
		}
		pEv = pEv->GetPrev();
	}
	return pMsN;
}

short int wholeLUT[12] = {
	1,	///C
	1,	///C#
	0,	//D
	0,	//D#
	1,	//E,
	0,	//F
	0,	//F#
	1,	//G
	1,	//G#
	0,	//A
	0,	//A#
	1	//B
};

int CMsNote::WholeRestOffset()
{
	int n = m_Pitch % 12;
	int o = m_Pitch / 12;
	int rV;
	if(!(o & 1))
	{
		if(wholeLUT[n])
			rV = 4;
		else
			rV = -4;
	}
	else
	{
		if(wholeLUT[n])
			rV = -4;
		else
			rV = 4;
	}
	return rV;
}

int CMsNote::HalfRestOffset()
{
	int n = m_Pitch % 12;
	int o = m_Pitch / 12;
	int rV;
	if(o & 1)
	{
		if(wholeLUT[n])
			rV = 4;
		else
			rV = -4;
	}
	else
	{
		if(wholeLUT[n])
			rV = -4;
		else
			rV = 4;
	}
	return rV;
}

void CMsNote::ChangeRestColor(CDC *pDC,COLORREF newcolor,int w,int h)
{
	int i,j;
	COLORREF c;

	for(i=0;i<w;++i)
	{
		for(j=0;j<h;++j)
		{
			if(c = pDC->GetPixel(i,j))
				pDC->SetPixel(i,j,newcolor);
		}
	}
}

//------------------------------------
// Byte 0 Midi Status (NOTEON,NOTEOFF)
// Byte 1 Midi Note
// Byte 2 Velocity
//-------------------------------------
int RangeLUT[6] = { 0,-24,-12,0,12,24 };

void CMsNote::NoteOn(int velocity, CMsKeySignature *pKS)
{
	DWMSG dwMsg;
	int chan = m_pMidiInfo->GetChannel(m_Track);
	int note = m_Pitch + RangeLUT[m_pMidiInfo->GetRange(m_Track)];
	dwMsg.mC[2] = (char)velocity;
	dwMsg.mC[1] = (char)pKS->GetKeySigCorrection(note,this->m_Key);
	dwMsg.mC[0] = MIDI_NOTEON | (chan & 0x0f);
	midiOutShortMsg(m_hMidiOut,dwMsg.m_dwMsg);
	AfxGetApp()->m_pMainWnd->PostMessageA(WM_PLAYER, PLAYER_SONG_NOTEONOFF, dwMsg.m_dwMsg);
}

void CMsNote::NoteOff(int velocity, CMsKeySignature *pKS)
{
	DWMSG dwMsg;

	int chan = m_pMidiInfo->GetChannel(m_Track);
	int note = m_Pitch + RangeLUT[m_pMidiInfo->GetRange(m_Track)];

	dwMsg.mC[2] = (char)velocity;
	dwMsg.mC[1] = (char)pKS->GetKeySigCorrection(note,this->m_Key);
//	if(velocity == 0)
//		dwMsg.mC[0] = MIDI_NOTEON | (chan & 0x0f);
//	else
		dwMsg.mC[0] = MIDI_NOTEOFF  | (chan & 0x0f);
	midiOutShortMsg(m_hMidiOut,dwMsg.m_dwMsg);
	AfxGetApp()->m_pMainWnd->PostMessageA(WM_PLAYER, PLAYER_SONG_NOTEONOFF, dwMsg.m_dwMsg);
}

int NoteInc[12] = {
	2,	//C
	2,	//C#
	2,	//D
	2,	//D#
	1,	//E
	2,	//F
	2,	//F#
	2,	//g
	2,	//G#
	2,	//A
	2,	//A#
	1	//B
};

void CMsNote::IncrNote()
{
	int Pitch = this->m_Pitch;
	int i = Pitch % 12;
	Pitch += NoteInc[i];
	m_Pitch = Pitch;
}

int NoteDec[12] = {
	1,	//C
	1,	//C#
	2,	//D
	2,	//D#
	2,	//E
	1,	//F
	1,	//F#
	2,	//g
	2,	//G#
	2,	//A
	2,	//A#
	2	//B
};

void CMsNote::DecrNote()
{
	int Pitch = this->m_Pitch;
	int i = Pitch % 12;
	Pitch -= NoteDec[i];
	m_Pitch = Pitch;
}

CMsObject * CMsNote::Copy()
{
	CMsObject *pOb = 0;
	CMsNote *pN = new CMsNote();
	pN->SetAccent(IsAccent());
	pN->SetDuration(GetDuration());
	pN->SetKey(GetKey());
	pN->SetLegato(IsLegato());
	pN->SetPitch(GetPitch());
	pN->SetRest(IsRest());
	pN->SetStacato(IsStacato());
	pN->SetTieBeg(IsTieBeg());
	pN->SetTieEnd(IsTieend());
	pN->SetTrack(GetTrack());
	pN->SetUpsideDown(IsUpsideDown());
	pN->m_hMidiOut = m_hMidiOut;
	pN->m_pEvParent = m_pEvParent;
	pN->m_pMidiInfo = m_pMidiInfo;
	
	pOb = pN;
	return pOb;
}

void CMsNote::Save(FILE *pO)
{
	int Byte1=0,Byte2=0,Byte3=0;
	//-------------------------
	// First Byte
	//-------------------------
	if(this->m_TieBeg) Byte1 |= MS_BEGTIE_FLAG;
	if(this->m_TieEnd) Byte1 |= MS_ENDTIE_FLAG;
	if(this->m_Rest) Byte1 |= MS_REST_FLAG;
	Byte1 |= this->m_Track;
	//-------------------------
	// Second Byte
	//-------------------------
	if(this->m_Accent) Byte2 |= MS_ACCENT_FLAG;
	Byte2 |= this->m_Duration;
	Byte2 |= this->m_Key << MS_KEY_SHIFT;
	//--------------------------
	// third Byte
	//--------------------------
	Byte3 = this->m_Pitch;
	if(this->m_UpsideDown) Byte3 |= MS_NOTE_UPSIDE_DOWN;
	fputc(Byte1,pO);
	fputc(Byte2,pO);
	fputc(Byte3,pO);
}

bool CMsNote::IsDotted()
{
	return DurTab[m_Duration].Dotted?true:false;
}
