// MsNote.h: interface for the CMsNote class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MSNOTE_H__5101728D_EA2E_4904_9DE5_4F2F714DBEAF__INCLUDED_)
#define AFX_MSNOTE_H__5101728D_EA2E_4904_9DE5_4F2F714DBEAF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MsObject.h"
#include "MsEvent.h"
#include "MidiInfo.h"
#include "MsKeySignature.h"
#include <mmsystem.h>

//---------------------------------
// Defines for "m_Key"
//---------------------------------
#define	MS_NOTE_INKEY	0
#define	MS_NOTE_NATURAL	1
#define MS_NOTE_SHARP	2
#define	MS_NOTE_FLAT	3

extern int IsOnLine(int pitch);


class CMsNote : public CMsObject
{
	// Indicates that the mouse is over this note
	unsigned m_Rest:1;
	unsigned m_TieBeg:1;
	unsigned m_TieEnd:1;
	unsigned m_Accent;
	unsigned m_Stacato:1;
	unsigned m_Legato:1;
	unsigned m_UpsideDown:1;
	unsigned m_Key:2;
	unsigned m_Track:4;	//value is 1->15, 0 is not used
	int m_Duration;
	int m_Pitch;
public:
	virtual void Save(FILE *pO);
	virtual CMsObject * Copy(void);
	void DecrNote(void);
	void IncrNote(void);
	CMsEvent * m_pEvParent;
	HMIDIOUT m_hMidiOut;
	CMidiInfo * m_pMidiInfo;
	void NoteOff(int velocity, CMsKeySignature *pKS);
	void NoteOn(int velocity, CMsKeySignature *pKS);
	void ChangeRestColor(CDC *pDC,COLORREF c,int w,int h);
	int HalfRestOffset();
	int WholeRestOffset();
	CMsNote * FindTieBegin();
	bool IsSolid(void);
	bool IsTriplet();
	int NeedsFlags(void);
	bool NeedsTail(void);
	int NeedsLine(void);
	virtual void Draw(CDC *pDC, int event, int maxevent);
	virtual void Print(FILE *pO);
	CMsNote();
	CMsNote(CMsEvent *pParent);
	virtual ~CMsNote();
	unsigned IsRest(void){return m_Rest;}
	void SetRest(unsigned v){m_Rest = v;}
	unsigned IsTieBeg(void){return m_TieBeg;}
	void SetTieBeg(unsigned v){m_TieBeg = v;}
	unsigned IsTieend(void){return m_TieEnd;}
	void SetTieEnd(unsigned v){m_TieEnd = v;}
	unsigned IsAccent(void){return m_Accent;}
	void SetAccent(unsigned v){m_Accent = v;}
	unsigned IsStacato(void){return m_Stacato;}
	void SetStacato(unsigned v){m_Stacato = v;}
	unsigned IsLegato(void){return m_Legato;}
	void SetLegato(unsigned v){m_Legato = v;}
	unsigned GetKey(void){return m_Key;}
	void SetKey(unsigned k){m_Key = k;}
	unsigned GetTrack(void){return m_Track;}
	void SetTrack(unsigned t){m_Track = t;}
	int GetDuration(void);
	void SetDuration(int d){m_Duration = d;}
	int GetPitch(void){return m_Pitch;}
	void SetPitch(int p){m_Pitch = p;}
	unsigned IsUpsideDown(void){return m_UpsideDown;}
	void SetUpsideDown(int u){m_UpsideDown = u;}
	int GetUpsideDown(void){return m_UpsideDown;}
	// This function converts the note value to a string
	int NoteToString(char * pInString,int size);
	bool IsDotted();
	int operator==(CMsNote &note);
};

#endif // !defined(AFX_MSNOTE_H__5101728D_EA2E_4904_9DE5_4F2F714DBEAF__INCLUDED_)
