// MsSong.cpp: implementation of the CMsSong class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MsSong.h"
#include "MsBar.h"
#include "MsEnd.h"
#include "MsKeySignature.h"
#include "MsLoudness.h"
#include "MsNote.h"
#include "MsObject.h"
#include "MsRepeatEnd.h"
#include "MsRepeatStart.h"
#include "Mstempo.h"
#include "MsTimeSig.h"
#include "Resource.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMsSong::CMsSong()
	: m_nIsPlaying(0)
{
	m_pEventListHead = 0;
	m_pEventListTail = 0;
	m_pEQ = 0;
	ResetBarCount();
	AddEventAtStart(new CMsEvent);
	m_pWndParent = 0;
	m_hMidiOut = 0;
	m_nTotalEvents=0;

}

CMsSong::~CMsSong()
{
	if(m_pEventListHead)
	{
		CMsEvent *pEv,*pEvDel;
		pEv = m_pEventListHead;
		while(pEv)
		{
			pEvDel = pEv;
			pEv = pEv->GetNext();
			delete pEvDel;
		}
	}
	if(m_pEQ)
	{
		delete m_pEQ;
	}
}

void CMsSong::AddEventAtEnd(CMsEvent *pE)
{
	if(m_pEventListHead == 0)
	{
		m_pEventListHead = pE;
		m_pEventListTail = pE;
	}
	else
	{
		pE->SetPrev(m_pEventListTail);
		m_pEventListTail->SetNext(pE);
		m_pEventListTail = pE;
	}
	m_nTotalEvents++;
}

void CMsSong::AddEventAtStart(CMsEvent *pE)
{
	if(m_pEventListHead == 0)
	{
		m_pEventListHead = pE;
		m_pEventListTail = pE;
	}
	else
	{
		pE->SetNext(m_pEventListHead);
		m_pEventListHead->SetPrev(pE);
		m_pEventListHead = pE;
	}
	m_nTotalEvents++;
}

CMsEvent * CMsSong::GetEventHead()
{
	return this->m_pEventListHead;
}

int CMsSong::Parse(char *pSongData)
{
	/***********************************************
	** Parse
	**		This function is used to parse
	** the raw data of the Music Studio Song
	** file data to create a data base
	**
	**	parameter:
	**		pSongData....pointer to the song data
	************************************************/
	int rV=0;
	int loop = 1;
	int c;
	int i=0;
	int Event=0;

	int KeySig,Tempo,TimeSig,Loudness,Repeat,t1,t2;
	ObjectTypes obj;
	obj.pObj = 0;
	CMsEvent *pEv = new CMsEvent;
	pEv->m_Index = Event++;
//	double MsPerTick;

	while(loop)
	{
		c = (int)pSongData[i++] & 0xff;
		obj.pObj = 0;
		switch(c)	/*	switch song data	*/
		{
			case TOKEN_REPEAT_STOP:	//basically the same thing as an End Event
				obj.pRepE = new CMsRepeatEnd;
				break;
			case TOKEN_EVENT_END:
				this->AddEventAtEnd(pEv);
				pEv = new CMsEvent;
				pEv->m_Index = Event++;
				break;
			case TOKEN_KEY_SIGNATURE:
				KeySig = (int)pSongData[i++] & 0xff;
				KeySig = (int)pSongData[i++] & 0xff;
				obj.pKey = new CMsKeySignature(KeySig);
				break;
			case TOKEN_TEMPO:
				Tempo = (int)pSongData[i++] & 0xff;
				obj.pTemp = new CMsTempo(int(Tempo));
				break;
			case TOKEN_BAR:	//don't do much of anything
				obj.pBar = new CMsBar;
				break;
			case TOKEN_TIME_SIGNATURE:
				TimeSig = (int)pSongData[i++] & 0xff;
				obj.pTime = new CMsTimeSig(TimeSig);
				break;
			case TOKEN_LOUDNESS:
				Loudness = (int)pSongData[i++] & 0xff;
				Loudness = (int)pSongData[i++] & 0xff;
				obj.pLoud = new CMsLoudness(Loudness);
				break;
			case TOKEN_REPEAT_START:
				Repeat = (int)pSongData[i++] & 0xff;
				obj.pRepS = new CMsRepeatStart(Repeat);
				break;
			case TOKEN_END:
				obj.pEnd = new CMsEnd;
				loop = 0;
				rV = 0;
				break;
			default:
				/*
					if we got down here, what was found must be a note
					so, we first get the track/type,duration,accidentals
					and then the note number
					If, the note starts a tie, treat it like any other
					note, if a note is at the end of a tie, then find
					the note in the time out buffer.  If the note is
					the beginning and end of a tie, still find it
					in the timeout buffer
				*/
				t1 = (int)pSongData[i++] & 0xff;	/*	duration	*/
				t2 = (int)pSongData[i++] & 0xff;	/*	note mask off MSB (upside down note)	*/
				obj.pNote = new CMsNote;
				obj.pNote->m_pMidiInfo = this->m_pMidiInfo;
				obj.pNote->m_hMidiOut = this->m_hMidiOut;
				obj.pNote->SetAccent((t1 & MS_ACCENT_FLAG)?1:0);
				obj.pNote->SetDuration(t1 & MS_DURATION_MASK);
				obj.pNote->SetKey((t1 & MS_KEY_MASK) >> MS_KEY_SHIFT);
				obj.pNote->SetLegato(0);		//Not supported
				obj.pNote->SetPitch(t2 & 0x7f);
				obj.pNote->SetRest((c & MS_REST_FLAG)?1:0);
				obj.pNote->SetStacato(0);		//Not supported
				obj.pNote->SetTieBeg((c & MS_BEGTIE_FLAG)?1:0);
				obj.pNote->SetTieEnd((c & MS_ENDTIE_FLAG)?1:0);
				obj.pNote->SetTrack((c & MS_TRACK_MASK));
				obj.pNote->SetUpsideDown((t2 & MS_NOTE_UPSIDE_DOWN)?1:0);
				obj.pNote->m_pEvParent = pEv;
				break;
		}	/*	end of switch c	*/
		if(obj.pObj)
		{
			pEv->AddObjectAtEnd(obj.pObj);
//			obj.pObj->Print(stdout);
		}
	}	/*	end while	*/
	RenumberEvents(NULL,NULL);
	return rV;
}


void CMsSong::Print(FILE *pO)
{
	CMsEvent *pEv = this->GetEventHead();
	fprintf(pO, "Song Total Events:%d\n", m_nTotalEvents);
	while(pEv)
	{
		pEv->Print(pO);
		fflush(pO);
		pEv = pEv->GetNext();
	}
}

int CMsSong::Play(HMIDIOUT hMo, CWnd *pW, CWnd *pStaffView)
{
	int rV = 1;
	if (!m_nIsPlaying)
	{
		m_pEQ = new CEventQueue();
		m_pEQ->SetWindow(pW);
		m_pEQ->SetStaffView(pStaffView);
		m_pEQ->SetMidiInfo(this->m_pMidiInfo);
		m_pEQ->SetSongBegin(m_pEventListHead);
		m_pEQ->SetMidiOut(hMo);
		m_nIsPlaying = 1;
		m_pEQ->Start();
		m_pEQ->ProcessEvent();
	}
	else
		rV = 0;
    return rV;
}

void CMsSong::CreateMidiInfo()
{
    m_pMidiInfo = new CMidiInfo;
	m_pMidiInfo->Create();
}

void CMsSong::CreateMidiInfo(char *pInfo)
{
    m_pMidiInfo = new CMidiInfo;
	m_pMidiInfo->Create(pInfo);
}

unsigned short CMsSong::LittleEndian(unsigned short bE)
{
    union {
        unsigned char m_C[2];
        unsigned short m_S;
    }CV;
    unsigned char t;

    CV.m_S = bE;
    t = CV.m_C[0];
    CV.m_C[0] = CV.m_C[1];
    CV.m_C[1] = t;
    return CV.m_S;
}

void CMsSong::Draw(CDC *pDC, int event, int maxevent,CRect *pRect)
{
	int i;
	int MaxX = pRect->right;
	CPen blk,*oldpen;
	blk.CreatePen(PS_SOLID,1,RGB(0,0,0));
	oldpen = pDC->SelectObject(&blk);
	//------------------------------------
	// draw staves
	//------------------------------------
	for(i=0;i<5;++i)	//trebel
	{
		pDC->MoveTo(0,i*8+STAVE_OFFSET);
		pDC->LineTo(MaxX,i*8+STAVE_OFFSET);
	}
	int o = i * 8 + STAVE_OFFSET + 8;
	for(i=0;i<5;++i)	//bass
	{
		pDC->MoveTo(0,i*8+o);
		pDC->LineTo(MaxX,i*8+o);
	}
	//-----------------------------
	// draw clefs
	//-----------------------------
	CBitmap Treb,Bass;

	Treb.LoadBitmap(IDB_BITMAP_TREBLE);
	Bass.LoadBitmap(IDB_BITMAP_BASS);
	CDC memDC;
	memDC.CreateCompatibleDC(pDC);
	memDC.SelectObject(&Treb);
	pDC->BitBlt(2,STAVE_OFFSET-4,16,48,&memDC,0,0,SRCAND);
	memDC.SelectObject(&Bass);
	pDC->BitBlt(2,STAVE_OFFSET+46,16,48,&memDC,0,0,SRCAND);

	pDC->SelectObject(oldpen);
	CMsEvent *pEv = this->GetEventObject(event);
	i = 0;
	for(i=0;(i<maxevent) && pEv;++i,pEv = pEv->GetNext())
	{
		if(pEv)pEv->Draw(pDC,i,maxevent);
	}
}

int CMsSong::AddObjectToSong(int event, CMsObject *pO)
{
	CMsEvent *pList = this->m_pEventListHead;
	CMsObject *rpO;
	int loop = 1;
	CMsObject *pObj;
	int rV = 1;		//indicate object was added

	while(loop && pList)
	{
		if ((pObj = pList->ContainsObjectType(MSOBJ_END)))
			pList->RemoveObject(pObj);
		if(pList->m_Index == event)
			loop = 0;
		else
			pList = pList->GetNext();
	}
	if(pList)	///found event, add object
	{
		if((rpO = pList->ObjectAlreadyHere(pO)) != NULL)
			pList->RemoveObject(rpO);
		else
			pList->AddObjectAtEnd(pO);
	}
	else
	{
		///we did not find event, so add events
		///until we get up to the right place
		do
		{
			pList = new CMsEvent();
			AddEventAtEnd(pList);
		}while(pList->m_Index < event);
		pList->AddObjectAtEnd(pO);
	}
	return rV;
}

void CMsSong::RenumberEvents(int *First, int *Last)
{
	int SelFlag = 0;
	int i = 0;
	m_nTotalEvents = 0;
	CMsEvent *pEv = this->GetEventHead();
	
	while(pEv)
	{
		m_nTotalEvents++;
		pEv->m_Index = i++;
		if(pEv->IsSelected() && !SelFlag)
		{
			SelFlag = 1;
			if(First) *First = pEv->m_Index;
		}
		else if (pEv->IsSelected() && SelFlag)
		{
			if(pEv->GetNext())
			{
				if(!pEv->GetNext()->IsSelected())
					if(Last)*Last = pEv->m_Index;
			}
			else
			{
				if(Last) *Last = pEv->m_Index;
			}
		}
		pEv = pEv->GetNext();
	}
	SetEventCount(m_nTotalEvents);
}

CMsEvent *CMsSong::InsertEvent(int e)
{
	///--------------------------------
	///	InsertEvent
	///
	///		This function inserts an
	///	empty event at the given
	///	position.  If there are no
	/// events at that position, then
	/// this function will create
	/// them until there is.
	///
	/// parameters:
	///		e.....event position to insert new event
	///
	/// return value:
	///		returns a pointer to the new
	///	event.
	///-------------------------------------
	CMsEvent *pEv = GetEventHead();
	CMsEvent *pNewEv=0;

	int loop = 1;

	while(pEv && loop)
	{
		if(pEv->m_Index == e)	// find event position?
			loop = 0;	// yes break from loop
		else
		{
			if (NULL == pEv->GetNext())
			{
				//----------------------------
				// there are no more events
				// so we need to create them
				//----------------------------
				int i;
				for (i = pEv->m_Index; i <= e; ++i)
				{
					// create the needed number of events
					pEv = new CMsEvent;
					AddEventAtEnd(pEv);
					m_nTotalEvents++;
				}
				RenumberEvents(NULL,NULL);
				pNewEv = pEv;
				pEv = 0;
				loop = 0;
			}
			else
				pEv = pEv->GetNext();
		}
	}
	if(pEv)
	{
		pNewEv = new CMsEvent;
		pNewEv->SetPrev(pEv);
		pNewEv->SetNext(pEv->GetNext());
		pEv->SetNext(pNewEv);
		pNewEv->GetNext()->SetPrev(pNewEv);
		m_nTotalEvents++;
	}
	return pNewEv;
}

CMsNote * CMsSong::CheckForNotePresence(int Event, int Note)
{
	//-------------------------------
	// CheckForNotePresence
	//	This function is used to find
	// out if there is a particular 
	// note at a certain location
	//
	//	parameters:
	//		Event....Event to look in
	//		Note.....Note to check for
	//
	// returns NULL if not found,
	//	otherwise, returns point to note
	//------------------------------------

	CMsNote *pN=0;
	CMsObject *pOb;
	CMsEvent *pEv = this->GetEventHead();
	int loop = 1;
	while(pEv && loop)
	{
		if(pEv->m_Index == Event)
			loop = 0;
		else
			pEv = pEv->GetNext();
	}
	if(pEv)
	{
		pOb = pEv->GetHead();
		loop = 1;
		while(loop && pOb)
		{
			if(pOb->GetType() == MSOBJ_NOTE)
			{
				CMsNote *pNC = (CMsNote *)pOb;
				if(!pNC->IsRest() && pNC->GetPitch() == Note)
				{
					pN = pNC;
					loop = 0;
				}
			}
			if(pN == 0)
				pOb = pOb->GetNext();
		}
	}
	return pN;
}

CMsEvent *CMsSong::GetEventObject(int Event)
{
	//------------------------------------
	// GetEventObject
	//	This function takes an Event
	// number and turns it into the
	// appropriate pointer to an Event
	// object.  If there is no object
	// there yet, it returns NULL
	// 
	// parameter:
	//		Event....Index of desired event
	//
	//	returns: pointer to the CMsEvent
	//			NULL if not found
	//-------------------------------------
	CMsEvent *pEv = this->GetEventHead();
	int loop = 1;
	while(pEv && loop)
	{
		// Is this the one?
		if(pEv->m_Index == Event)
			loop = 0;	//Yes, stop seraching
		else
			// Keep seraching
			pEv = pEv->GetNext();
	}
	return pEv;
}

//--------------------------------------------
// The following is what I will consider to be
// the default Atari sound data.  It was taken
// from one of the song files that came on the 
// original disk.
//---------------------------------------------

int AtariInst[270] = {
0x42,0x6c,0x6f,0x63,0x6b,0x73,
0x00,0x00,0x00,0x00,0x48,0x61,0x72,0x6d,
0x6f,0x6e,0x69,0x63,0x61,0x00,0x47,0x75,
0x69,0x74,0x61,0x72,0x00,0x00,0x00,0x00,
0x46,0x6c,0x75,0x74,0x65,0x00,0x00,0x00,
0x00,0x00,0x43,0x6c,0x61,0x72,0x69,0x6e,
0x65,0x74,0x00,0x00,0x42,0x61,0x72,0x69,
0x74,0x6f,0x6e,0x65,0x00,0x00,0x48,0x69,
0x68,0x61,0x74,0x00,0x6f,0x00,0x6e,0x00,
0x53,0x6e,0x61,0x72,0x65,0x00,0x6f,0x00,
0x00,0x00,0x42,0x2e,0x46,0x69,0x64,0x64,
0x6c,0x65,0x00,0x00,0x53,0x61,0x78,0x00,
0x74,0x6f,0x6e,0x65,0x00,0x00,0x43,0x6c,
0x61,0x72,0x69,0x6e,0x65,0x74,0x00,0x00,
0x50,0x69,0x61,0x6e,0x6f,0x00,0x00,0x00,
0x00,0x00,0x42,0x61,0x73,0x73,0x00,0x73,
0x00,0x00,0x00,0x00,0x56,0x69,0x62,0x65,
0x73,0x00,0x65,0x74,0x00,0x00,0x42,0x65,
0x6c,0x6c,0x73,0x00,0x00,0x00,0x00,0x00,
0x8f,0x23,0x0c,0x06,0x07,0x08,0x06,0x09,
0x86,0x22,0x0c,0x05,0x0b,0x00,0x05,0x00,
0x80,0x12,0x0d,0x03,0x07,0x04,0x05,0x02,
0x82,0x14,0x0a,0x08,0x0c,0x08,0x08,0x07,
0x8f,0x29,0x0d,0x06,0x0c,0x08,0x0a,0x0c,
0x85,0x38,0x0a,0x0e,0x09,0x0a,0x07,0x04,
0x1e,0x03,0x0d,0x05,0x05,0x03,0x05,0x05,
0x1e,0x46,0x0d,0x0f,0x07,0x0f,0x04,0x0f,
0x85,0x43,0x0e,0x07,0x08,0x0e,0x07,0x01,
0x8d,0x34,0x0e,0x0f,0x08,0x04,0x05,0x07,
0x82,0x46,0x08,0x07,0x06,0x0a,0x0b,0x0a,
0x80,0x22,0x0e,0x07,0x08,0x05,0x06,0x00,
0x8f,0x32,0x0e,0x05,0x0b,0x08,0x0a,0x0f,
0x84,0x03,0x0f,0x05,0x09,0x03,0x00,0x01,
0x8f,0x01,0x0c,0x01,0x08,0x02,0x07,0x00
};


void CMsSong::Save(FILE *pO)
{
	int NoEnd = 1;
	//---------------------------------------
	// Before we save, make sure that there
	// is an END object at the end of the 
	// song.
	//---------------------------------------
	if(this->m_pEventListTail)
	{
		CMsObject *pOb = m_pEventListTail->GetHead();
		while(pOb && NoEnd)
		{
			if(MSOBJ_END == pOb->GetType())
				NoEnd = 0;
			pOb = pOb->GetNext();
		}
	}
	//-------------------------------------
	// If there is no End, Add one.
	//-------------------------------------
	if(NoEnd)
	{
		CMsEnd *pME = new CMsEnd;
		this->AddEventAtEnd(new CMsEvent);
		this->AddEventAtEnd(new CMsEvent);
		m_pEventListTail->AddObjectAtEnd(pME);
		this->RenumberEvents(NULL,NULL);
	}
	//-----------------------------------
	// Save
	// this function is used to save a 
	// song file
	//
	// parameter:
	//		pO.....pointer to output stream
	//-----------------------------------
	fputc(0xcd,pO);	//first character of file
	fprintf(pO,"Mstudio");
	fputc(0xcd,pO);
	fputc(0x02,pO);
	//------------------------------------
	// Next, we have the default atari
	// instrument data.  I just copied
	// this from a random Music Studio
	// file, as there is not a whole lot
	// that I can do with it.
	//------------------------------------
	int i;

	for(i=0;i<270;++i)
		fputc(AtariInst[i],pO);
	//-----------------------------------
	// Save Midi Info
	//-----------------------------------
	GetMidiInfo()->Save(pO);
	//-----------------------------------
	// Save song content
	//-----------------------------------
	CMsEvent *pEv = this->GetEventHead();
	while(pEv)
	{
		CMsObject *pObj;
		pObj = pEv->GetHead();
		while(pObj)
		{
			pObj->Save(pO);
			pObj = pObj->GetNext();
		}
		fputc(0,pO);	//event delemiter
		pEv = pEv->GetNext();
	}
}


// Finds out if there is a note below the cursor
bool CMsSong::IsNoteInThisEvent(int Note, int Event)
{
	bool rV = false;
	CMsObject *pObj;
	CMsNote *pN;
	CMsEvent *pEv = this->GetEventObject(Event);
	if (pEv)
	{
		pObj = pEv->GetHead();
		int loop = 1;
		while (pObj && loop)
		{
			if (pObj->GetType() == MSOBJ_NOTE)
			{
				pN = (CMsNote *)pObj;
				if (Note == pN->GetPitch())
				{
					rV = true;
					loop = 0;
				}
				else
					pObj = pObj->GetNext();
			}
			else
				pObj = pObj->GetNext();
		}
	}
	return rV;
}


// Gets a pointer to the note at the given event position
CMsNote * CMsSong::GetNoteAtEvent(int Note, int Event)
{
	CMsNote *rV = 0;
	CMsObject *pObj;
	CMsNote *pN;
	CMsEvent *pEv = this->GetEventObject(Event);
	if (pEv)
	{
		pObj = pEv->GetHead();
		int loop = 1;
		while (pObj && loop)
		{
			if (pObj->GetType() == MSOBJ_NOTE)
			{
				pN = (CMsNote *)pObj;
				if (Note == pN->GetPitch())
				{
					rV = pN;
					loop = 0;
				}
				else
					pObj = pObj->GetNext();
			}
			else
				pObj = pObj->GetNext();
		}
	}
	return rV;
}


// Removes an event from the song
void CMsSong::RemoveEvent(int Event)
{
	CMsEvent *pEv = GetEventObject(Event);
	if(pEv) DeleteEvent(pEv);
}


// Delete Event object
void CMsSong::DeleteEvent(CMsEvent * pEvent)
{
	if (pEvent == this->m_pEventListHead)
	{
		m_pEventListHead = pEvent->GetNext();
		if (m_pEventListHead)
			m_pEventListHead->SetPrev(0);
		else
			m_pEventListTail = 0;
	}
	else if (pEvent == this->m_pEventListTail)
	{
		m_pEventListTail = pEvent->GetPrev();
		if (m_pEventListTail)
			m_pEventListTail->SetNext(0);
		else
			m_pEventListHead = 0;
	}
	else
	{
		pEvent->GetNext()->SetPrev(pEvent->GetPrev());
		pEvent->GetPrev()->SetNext(pEvent->GetNext());
	}
	RenumberEvents(0, 0);
	delete pEvent;
}


// Stops the song from playing
int CMsSong::Stop()
{
	if (m_nIsPlaying == 1)
	{
		if (m_pEQ)
		{
			m_pEQ->Stop();
			m_nIsPlaying = 2;
		}
	}
	return 0;
}

int CMsSong::IsRepeatInThisEvent(int nEvent)
{
	int rV = 0;
	CMsEvent *pEV = GetEventObject(nEvent);
	if (pEV)
	{
		CMsObject *pObj = pEV->GetHead();
		while (pObj && !rV)
		{
			int type;
			type = pObj->GetType();
			if (type == MSOBJ_REPEATSTART || type == MSOBJ_REPEATEND)
				rV = 1;
			else
				pObj = pObj->GetNext();
		}
	}
	return rV;
}

CMsObject * CMsSong::GetRepeatObject(int nEvent)
{
	int rV = 0;
	CMsEvent *pEV = GetEventObject(nEvent);
	CMsObject *pObj = pEV->GetHead();
	while (pObj && !rV)
	{
		int type;
		type = pObj->GetType();
		if (type == MSOBJ_REPEATSTART || type == MSOBJ_REPEATEND)
			rV = 1;
		else
			pObj = pObj->GetNext();
	}
	return pObj;
}

CMsObject * CMsSong::FindMatchingRepeatObject(CMsObject * pRepeatObject, int nEvent, int * pnMatchingEvent)
{
	//-------------------------------
	// FindMatchingRepeatObject
	//		This is not quite as easy
	// as it sounds.  Repeats can
	// be nested, so as we scan we
	// need to keep track of the nesting
	//
	// parameters:
	//	pRepeatObject...find match to this object
	//	nEvent..........Event above object is in
	//	pnMatchingEvent.pointer to event where match is
	// Return Value:
	//		Returns 0 on fail
	//		Returns pointer of match on success
	//-------------------------------
	int Type = pRepeatObject->GetType();
	CMsEvent *pEV = GetEventObject(nEvent);
	int loop = 1;
	int Nesting = 0;
	CMsObject *pObj=0;

	if (MSOBJ_REPEATEND == Type) pEV = pEV->GetPrev();
	else if (MSOBJ_REPEATSTART == Type) pEV = pEV->GetNext();
	while (loop && pEV)
	{
		if ((pObj = pEV->ContainsRepeatObject()) != nullptr)
		{
			if (MSOBJ_REPEATSTART == Type)
			{
				if (MSOBJ_REPEATEND == pObj->GetType())
				{
					if (Nesting == 0)
						loop = 0;
					else
						Nesting--;
				}
				else
				{
					Nesting++;
				}
			}
			else  //MSOBJ_REPEATEND
			{
				if (MSOBJ_REPEATSTART == pObj->GetType())
				{
					if (Nesting == 0)
						loop = 0;
					else
						Nesting--;
				}
				else
				{
					Nesting++;
				}

			}
		}
		else
		{
			if (MSOBJ_REPEATEND == Type) pEV = pEV->GetPrev();
			else if (MSOBJ_REPEATSTART == Type) pEV = pEV->GetNext();
		}
	}
	return pObj;
}

int CMsSong::IsKeySigInThisEvent(int nEvent)
{
	CMsEvent *pEV;
	int rV = 0;

	pEV = this->GetEventObject(nEvent);
	if (pEV)
	{
		if (pEV->ContainsObjectType(MSOBJ_KEYSIG)) rV = 1;
	}
	return rV;
}

int CMsSong::IsTimeSigInThisEvent(int nEvent)
{
	CMsEvent *pEV;
	int rV = 0;

	pEV = this->GetEventObject(nEvent);
	if (pEV)
	{
		if(pEV->ContainsObjectType(MSOBJ_TIMESIG)) rV = 1;
	}
	return rV;
}


CMsObject * CMsSong::GetObjectTypeInEvent(int nType, int nEvent)
{
	CMsEvent *pEV = GetEventObject(nEvent);
	CMsObject *pObj = 0;
	if (pEV) pObj = pEV->ContainsObjectType(nType);
	return pObj;
}


int CMsSong::IsTempInThisEvent(int nEvent)
{
	CMsEvent *pEV;
	int rV = 0;

	pEV = this->GetEventObject(nEvent);
	if (pEV)
	{
		if (pEV->ContainsObjectType(MSOBJ_TEMPO)) rV = 1;
	}
	return rV;
}


int CMsSong::IsLoudnessInThisEvent(int nEvent)
{
	CMsEvent *pEV;
	int rV = 0;

	pEV = this->GetEventObject(nEvent);
	if (pEV)
	{
		if (pEV->ContainsObjectType(MSOBJ_LOUDNESS)) rV = 1;
	}
	return rV;
}


CMsObject * CMsSong::IsObjectInEvent(int nEvent, int nObjectType, int &flag)
{
	CMsEvent *pEV;
	CMsObject *pObj = 0;
	int rV = 0;

	pEV = this->GetEventObject(nEvent);
	if (pEV)
	{
		pObj = pEV->ContainsObjectType(nObjectType);
		if (pObj) flag = 1;
		else flag = 0;
	}
	else flag = 0;
	return pObj;
}
