#ifndef MUSICSTUDIOFILEFORMAT__H
#define MUSICSTUDIOFILEFORMAT__H

//----------------------------------
//
// File Format:
// Header:
// BYTE0:		$CD
// BYTE1-7		"Mstudio"
// BYTE 8-9		$CD-$02
// BYTE10-159		Atari ST instrument names
//				This is an array of 15 names
//				Each name is 10 bytes
// BYTE 160-279	Atari instrument settings.
//				This is an array of 15 blocks
//				of 8 bytes each which correspond
//				to each of the 15 instruments
// BYTE 280-429	Instrument names for MIDI devices
//				This is an array of 15 blocks of 10
//				bytes each.
// BYTE 430-444	An array of 15 bytes containing the
//				midi channel and the range
// BYTE 445-459 An array of 15 bytes that contain the
//				patch number for the instrument
// BYTE 460-479 Displacements to the lyric lines
// BYTE 480-511	32 Byte Song Title
// End of header
// BYTE 512-519 These are evidently the track enable words
// BYTE 520     This is where the actual song data starts
//              Why do we need four sets of track enables?
//-----------------------------------------------------

//Offset Defines
#define MS_MIDIINSTURMENTS_OFFSET		280
#define MS_SONGDATA_OFFSET				520

//-------------------------------------------
// time Signature Values
//-------------------------------------------

#define TIMESIG_2_2		1
#define TIMESIG_3_2		2
#define TIMESIG_2_4		3
#define TIMESIG_3_4		4
#define TIMESIG_4_4		5
#define TIMESIG_5_4		6
#define TIMESIG_6_8		7

//-----------------------------------------------------
// values for creating note dur from combo boxes

#define WHOLE_NOTE		18
#define HALF_NOTE		15
#define QUARTER_NOTE	12
#define EIGTH_NOTE		9
#define SIXTEENTH_NOTE	6
#define THIRTYSEC_NOTE	3

#define DOTTED_NOTE		(2)
#define TRIPLET_NOTE	(-2)
//-----------------------------------------------------
// Note values....not really needed
//-----------------------------------------------------
#define NOTE_C		0
#define NOTE_D		2
#define NOTE_E		4
#define NOTE_F		5
#define NOTE_G		7
#define NOTE_A		9
#define NOTE_B		11

#define NOTE_C2		0x24
#define NOTE_D2		0x26
#define NOTE_E2		0x28
#define NOTE_F2		0x29
#define NOTE_G2		0x2B
#define NOTE_A2		0x2D
#define NOTE_B2		0x2F
#define NOTE_C3		0x30

//------------------------------------------------------
// Key Signatures
//-----------------------------------------------------
#define CMAJ	0x01		//C major
#define	GMAJ	0x02		//G major
#define	DMAJ	0x03		//D major
#define	AMAJ	0x04		//A major
#define	EMAJ	0x05		//E major
#define	BMAJ	0x06		//B major
#define	FSMAJ	0x07		//F# major
#define	CSMAJ	0x08		//C# major
#define	FMAJ	0x09		//F major
#define	BFMAJ	0x0a		//Bb major
#define	EFMAJ	0x0b		//Eb mojor
#define	AFMAJ	0x0c		//Ab major
#define	DFMAJ	0x0d		//Db major
#define	GFMAJ	0x0e		//Gb major
#define	CFMAJ	0x0f		//Cb major?

//--------------------------------------
// Music Studio Token values for parser
//--------------------------------------

#define TOKEN_EVENT_END			0x0
#define TOKEN_KEY_SIGNATURE		0x80
#define	TOKEN_TEMPO				0x81
#define	TOKEN_BAR				0x82
#define	TOKEN_TIME_SIGNATURE	0x83
#define	TOKEN_LOUDNESS			0x84
#define	TOKEN_REPEAT_START		0x85
#define	TOKEN_REPEAT_STOP		0x86
#define TOKEN_END				0xFF

//---------------------------
// defines for note tokens
//---------------------------

// BYTE 0
								//bit 0 track lsb
								//bit 1 track
								//bit 2 track
								//bit 3 track msb
#define MS_REST_FLAG	(0x10)	//bit 4
#define MS_ENDTIE_FLAG	(0x20)	//bit 5
#define MS_BEGTIE_FLAG	(0x40)	//bit 6
#define MS_TRACK_MASK	(0x0f)						//bit 7 not used

// BYTE 1

#define SHARP	0x80
#define	FLAT	0xc0
#define	INKEY	0x00
#define	NATURAL	0x40
#define MS_KEY_MASK	(0xc0)
#define MS_KEY_SHIFT	6
#define MS_ACCENT_FLAG	(0x20)
#define MS_DURATION_MASK	(0x1f)

// BYTE 2

#define MS_NOTE_UPSIDE_DOWN		0x80

//--------------------------------------
// Note Defines
//-------------------------------------

#define NOTE_C		0
#define NOTE_CS		1	//C sharp
#define NOTE_D		2
#define NOTE_DS		3	//D Sharp
#define NOTE_E		4
#define NOTE_F		5
#define NOTE_FS		6	//F sharp
#define NOTE_G		7
#define NOTE_GS		8	//G sharp
#define NOTE_A		9
#define NOTE_AS		10	//A sharp/B flat
#define NOTE_B		11

//---------------------------------
// accidental encoding
//---------------------------------

#define ACCIDENT_INKEY		0
#define ACCIDENT_NATURAL	1
#define ACCIDENT_SHARP		2
#define ACCIDNET_FLAT		3


#define REST_THIRTYSECOND		3
#define REST_SIXTEENTH			6
#define REST_EIGTH				9
#define REST_QUARTER			12
#define REST_HALF				15
#define REST_WHOLE				18

//------------------------
// Not really sure which
// duration goes with
// which note...(this
// may have been wrong
// to start with)
//------------------------
typedef struct {
	short NoteShapIndex;
	short Dotted;
	short Triplet;
	short Solid;
	short Flags;
	short Tail;
	short Dur;
	char *pName;
} DUR;

extern DUR DurTab[];

typedef struct {
	char Name[10];
} Instrument;

typedef struct {
	Instrument Name[15];		//Instrument names
	char chan_range[15];	/*	array contains data that coresponds to channel and range	*/
	char patch[15];			//	array that indicates midi patch for instrument
	long lyrics[5];			//	pointers to lyics
	char Title[32];			// Song Title
	unsigned short TrackEnables[4];
}MIDI_INFO;

#endif
