#ifndef WINDOWS_MESSAGES
#define WINDOWS_MESSAGES
//-------------------------------------
// Midi Messages
//----------------------------------
typedef union {
	DWORD	m_dwMsg;
	char	mC[4];
}DWMSG;

//----------------------------------
// Midi channel Messages
//----------------------------------
#define MIDI_NOTEOFF	(0x80)
#define MIDI_NOTEON		(0x90)
#define MIDI_POLYPRESS	(0xa0)
#define MIDI_CTRLCHNG	(0xb0)
#define MIDI_PGMCHANGE	(0xc0)
#define MIDI_CHNLPRESS	(0xd0)
#define MIDI_PITCHBEND	(0xe0)
//-------------------------------------
// Submessages for MIDI_CTRLCHNG
//-------------------------------------

#define MIDI_CTRLCHNG_ALLOFF	(120)
#define MIDI_CTRLCHNG_RESETALL	(121)
#define MIDI_CTRLCHNG_LOCAL		(122)
#define MIDI_CTRLCHNG_LOCAL_OFF	(0)		//sub message to MIDI_CTRLCHNG_LOCAL
#define MIDI_CTRLCHNG_LOCAL_ON	(127)	//sub message to MIDI_CTRLCHNG_LOCAL
#define MIDI_CTRLCHNG_NOTESOFF	(123)
#define MIDI_CTRLCHNG_OMNION	(124)
#define MIDI_CTRLCHNG_OMNIOFF	(125)
#define MIDI_CTRLCHNG_MONO_ON	(126)
#define MIDI_CTRLCHNG_POLY_ON	(127)

//-------------------------------------
// Midi Real Time Messages
//-------------------------------------
#define MIDI_CLOCK		(0xf8)
#define MIDI_START		(0xfa)
#define MIDI_STOP		(0xfc)
#define MIDI_CONTINUE	(0xfb)
//-------------------------------------
// Midi System Exclusive
//--------------------------------------
#define MIDI_SYSEX		(0xf0)
#define MIDI_SYSEXEND	(0xf7)

//---------------------------------------
// Controllers on the Launchkey 25
//--------------------------------------

#define MIDI_CTRLCHNG_KNOB_INSTRUMENT		21
#define MIDI_CTRLCHG_DECORATION				22
#define MIDI_CTRLCHNG_KNOB_NOTE_DUR			23
#define MIDI_CTRLCHNG_KNOB_REST_DUR			24
#define MIDI_CTRLCHNG_ACCIDENTAL			25
#define MIDI_CTRLCHNG_KEY					26

#define MIDI_CTRLCHNG_ROUNDUPPER			104
#define MIDI_CTRLCHGN_ROUNDLOWER			105

#define MIDI_CTRLCHNG_REWIND				112
#define MIDI_CTRLCHNG_FASTFORWARD			113
#define MIDI_CTRLCHNG_STOP					114
#define MIDI_CTRLCHNG_PLAY					115
#define MIDI_CTRLCHNG_PAUSE					116
#define MIDI_CTRLCHNG_RECORD				117
//------------------------------------------
// Midi channel defines for Launchkey25
//------------------------------------------
#define KEYBOARD_MIDI_CHANNEL	0	//midi channel for keyboard
#define BUTTON_MIDI_CHANNEL		9	//midi chanel of button pushes
//-----------------------------------------
// Note numbers for Buttons
//-----------------------------------------
#define BUTTON_UPPER_1			40
#define BUTTON_UPPER_ACCENT		41	//Button Upper 2
#define BUTTON_UPPER_3			42	//Button Upper 3
#define BUTTON_UPPER_4			43
#define BUTTON_UPPER_5			48
#define BUTTON_UPPER_6			49
#define BUTTON_UPPER_7			50
#define BUTTON_UPPER_8			51

#define BUTTON_LOWER_1			36
#define BUTTON_LOWER_2			37
#define BUTTON_LOWER_3			38
#define BUTTON_LOWER_4			39
#define BUTTON_LOWER_5			44
#define BUTTON_LOWER_6			45
#define BUTTON_LOWER_7			46
#define BUTTON_LOWER_8			47

//-----------------------------------
// Windows Messages
//-----------------------------------
#define WM_PLAYER		WM_USER+1
//submessages
#define PLAYER_SONG_BAR			0
#define PLAYER_SONG_END			1
#define PLAYER_SONG_ERROR		2
#define PLAYER_SONG_REPEAT		3
#define PLAYER_SONG_REPEAT_END	4
#define PLAYER_SONG_TEMPO		5
#define PLAYER_SONG_NOTE		6
#define PLAYER_SONG_PATCHCHNG	7
#define PLAYER_SONG_TICKDEBUG	8
#define PLAYER_SONG_EVENTEND	9
#define PLAYER_SONG_DISPEVENT	10
#define PLAYER_SONG_PLAYBUTTON	11
#define PLAYER_SONG_NOTEONOFF	12


#define WM_DRAWMODE			WM_USER+2

#define DRAW_NOP				0
#define DRAW_NOTE				1
#define DRAW_REST				2
#define DRAW_BAR				3
#define DRAW_TIE				4
#define DRAW_COPY				5
#define DRAW_MOVE				6
#define DRAW_REPEAT				7
#define DRAW_TEMPO				8
#define DRAW_TIMESIG			9
#define DRAW_KEYSIG				10
#define DRAW_LOUDNESS			11
#define DRAW_INSTCHANGE			12
#define DRAW_CHANGEDUR			13
#define DRAW_INCREASEDUR		14
#define DRAW_DECREASEDUR		15
#define DRAW_INCRPITCH			16
#define DRAW_DECRPITCH			17
#define DRAW_INSERTBLOCK		18
#define DRAW_DRAWNOTESVIAMIDI	19

//-----------------------------------
// DRAW_NOTE LPARAM Format
//-----------------------------------

#define DRAW_NOTE_DURATION		(0x1f)
#define DRAW_NOTE_ACCENT		(0x20)
#define DRAW_NOTE_ACCENT_SHIFT	(5)
#define DRAW_NOTE_KEY			(0x0c0)
#define DRAW_NOTE_KEY_SHIFT		(6)
#define DRAW_NOTE_TRACK			(0x0f00)
#define DRAW_NOTE_TRACK_SHIFT	(8)
#define DRAW_NOTE_REST			(0x1000)
#define DRAW_NOTE_REST_SHIFT	(12)
#define DRAW_NOTE_ENDTIE		(0x2000)	//not used
#define DRAW_NOTE_BEGTIE		(0x4000)	//not used

//--------------------------------------------
// Program change messages
//------------------------------------------------

#define WM_PGMCHANGE		WM_USER+3

//----------------------------------------
// Init Staff View
//------------------------------------

#define WM_INIT_STAFF		WM_USER+4

//------------------------------------
// Channel number change in Midi Setup
//-------------------------------------

#define WM_CHANCHANGE		WM_USER+5

//------------------------------------
// Midi In Message
//------------------------------------

#define WM_MIDIIN			WM_USER+6
#define WM_MIDIIN_LONG		WM_USER+7
#define WM_SHORTMIDIMSG		WM_USER+8

//-------------------------------------
// Message for CStaffView that tells
// where to highlight staff to show
// current song position
//-------------------------------------

#define WM_STAFF_DISP_EVENT		(WM_USER+9)

#define STAFF_DISP_EVENT_NEXT	0
#define STAFF_DISP_EVENT_END	1

#endif
